@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 # This file is part of Xpra.
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2013 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -222,14 +222,24 @@ def record_build_info(is_build=True):
     global BUILD_INFO_FILE
     props = get_properties(BUILD_INFO_FILE)
     if is_build:
-        try:
-            import getpass
-            set_prop(props, "BUILT_BY", getpass.getuser())
-        except:
-            set_prop(props, "BUILT_BY", os.environ.get("USER"))
-        set_prop(props, "BUILT_ON", socket.gethostname())
-        set_prop(props, "BUILD_DATE", datetime.date.today().isoformat())
-        set_prop(props, "BUILD_TIME", datetime.datetime.now().strftime("%H:%M"))
+        source_epoch = os.environ.get("SOURCE_DATE_EPOCH")
+        if source_epoch:
+            #reproducible builds:
+            build_time = datetime.datetime.utcfromtimestamp(int(source_epoch))
+            build_date = build_time.date()
+        else:
+            #win32, macos and older build environments:
+            build_time = datetime.datetime.now()
+            build_date = datetime.date.today()
+            #also record username and hostname:
+            try:
+                import getpass
+                set_prop(props, "BUILT_BY", getpass.getuser())
+            except:
+                set_prop(props, "BUILT_BY", os.environ.get("USER"))
+            set_prop(props, "BUILT_ON", socket.gethostname())
+        set_prop(props, "BUILD_DATE", build_date.isoformat())
+        set_prop(props, "BUILD_TIME", build_time.strftime("%H:%M"))
         set_prop(props, "BUILD_MACHINE", get_machineinfo())
         set_prop(props, "BUILD_CPU", get_cpuinfo())
         set_prop(props, "BUILD_BIT", platform.architecture()[0])
@@ -247,9 +257,9 @@ def record_build_info(is_build=True):
         #record pkg-config versions:
         PKG_CONFIG = os.environ.get("PKG_CONFIG", "pkg-config")
         for pkg in ("libc",
-                    "vpx", "libvpx", "x264", "x265",
+                    "vpx", "libvpx", "x264", "x265", "webp",
                     "avcodec", "avutil", "swscale",
-                    "nvenc7",
+                    "nvenc",
                     "x11", "xrandr", "xtst", "xfixes", "xkbfile", "xcomposite", "xdamage", "xext",
                     "gtk+-3.0", "pycairo", "pygobject-2.0", "pygtk-2.0", ):
             #fugly magic for turning the package atom into a legal variable name:
