@@ -1,9 +1,12 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2016-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os
+
+from xpra.os_util import get_hex_uuid
 from unit.server_test_util import ServerTestUtil, log
 
 uq = 0
@@ -17,12 +20,13 @@ class X11ClientTestUtil(ServerTestUtil):
 		return xvfb, self.do_run_client(client_display, *args)
 
 	def do_run_client(self, client_display, *args):
-		from xpra.scripts.server import xauth_add
-		xauth_add(client_display)
+		from xpra.x11.vfb_util import xauth_add
+		xauth_data = get_hex_uuid()
+		xauth_add(self.default_env["XAUTHORITY"], client_display, xauth_data, os.getuid(), os.getgid())
 		env = self.get_run_env()
 		env["DISPLAY"] = client_display
 		global uq
 		env["XPRA_LOG_PREFIX"] = "client %i: " % uq
-		uq +=1 
+		uq +=1
 		log("starting test client on Xvfb %s", client_display)
 		return self.run_xpra(["attach"] + list(args) , env)
