@@ -1,15 +1,13 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2016-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
-import sys
 import unittest
 
-from unit.client.x11_clipboard_test_util import X11ClipboardTestUtil
-from xpra.os_util import OSX
+from unit.client.x11_clipboard_test_util import X11ClipboardTestUtil, has_xclip
+from xpra.os_util import OSX, POSIX, PYTHON2
 
 from xpra.log import Logger
 log = Logger("clipboard")
@@ -37,7 +35,7 @@ class X11ClipboardTest(X11ClipboardTestUtil):
 
 
 def main():
-	if os.name=="posix" and sys.version_info[0]==2 and not OSX:
+	if POSIX and PYTHON2 and not OSX and has_xclip():
 		unittest.main()
 
 
