@@ -1,26 +1,33 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import hashlib
+from threading import Lock
+from PIL import Image
+
+from xpra.net.mmap_pipe import mmap_read
+from xpra.net import compression
+from xpra.util import typedict, csv, envint, envbool, repr_ellipsized
+from xpra.codecs.loader import get_codec, is_loaded
+from xpra.codecs.video_helper import getVideoHelper
+from xpra.os_util import BytesIOClass, bytestostr, _buffer
+try:
+    from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
+except ImportError:
+    from xpra.util import xor as xor_str
 from xpra.log import Logger
+
 log = Logger("paint")
 deltalog = Logger("delta")
 
-from threading import Lock
-from xpra.net.mmap_pipe import mmap_read
-from xpra.net import compression
-from xpra.util import typedict, csv, envint, envbool
-from xpra.codecs.loader import get_codec
-from xpra.codecs.video_helper import getVideoHelper
-from xpra.os_util import BytesIOClass, bytestostr, _buffer
-from xpra.codecs.xor.cyxor import xor_str   #@UnresolvedImport
-from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place   #@UnresolvedImport
-
 DELTA_BUCKETS = envint("XPRA_DELTA_BUCKETS", 5)
 INTEGRITY_HASH = envbool("XPRA_INTEGRITY_HASH", False)
+PAINT_BOX = envint("XPRA_PAINT_BOX", 0) or envint("XPRA_OPENGL_PAINT_BOX", 0)
+WEBP_PILLOW = envbool("XPRA_WEBP_PILLOW", False)
+
 
 #ie:
 #CSC_OPTIONS = { "YUV420P" : {"RGBX" : [swscale.spec], "BGRX" : ...} }
@@ -45,7 +52,7 @@ def load_video_decoders():
             specs = vh.get_decoder_specs(encoding)
             for colorspace, decoders in specs.items():
                 log("%-5s decoders for %7s: %s", encoding, colorspace, csv([d.get_type() for _,d in decoders]))
-                assert len(decoders)>0
+                assert decoders
                 #use the first one:
                 _, decoder_module = decoders[0]
                 VIDEO_DECODERS[encoding] = decoder_module
@@ -65,15 +72,18 @@ def fire_paint_callbacks(callbacks, success=True, message=""):
 
 """
 Generic superclass for all Backing code,
-see CairoBacking and GTKWindowBacking for actual implementations
+see CairoBackingBase and GTK2WindowBacking subclasses for actual implementations
 """
 class WindowBackingBase(object):
-    def __init__(self, wid, window_alpha, idle_add):
+    RGB_MODES = ()
+
+    def __init__(self, wid, window_alpha):
         load_csc_options()
         load_video_decoders()
         self.wid = wid
         self.size = 0, 0
-        self.idle_add = idle_add
+        self.render_size = 0, 0
+        self.offsets = 0, 0, 0, 0       #top,left,bottom,right
         self._alpha_enabled = window_alpha
         self._backing = None
         self._delta_pixel_data = [None for _ in range(DELTA_BUCKETS)]
@@ -81,14 +91,25 @@ class WindowBackingBase(object):
         self._csc_decoder = None
         self._decoder_lock = Lock()
         self._PIL_encodings = []
+        self.default_paint_box_line_width = PAINT_BOX or 1
+        self.paint_box_line_width = PAINT_BOX
         self.pointer_overlay = None
-        PIL = get_codec("dec_pillow")
-        if PIL:
-            self._PIL_encodings = PIL.get_encodings()
+        self.cursor_data = None
+        self.default_cursor_data = None
+        self.jpeg_decoder = None
+        self.webp_decoder = None
+        if is_loaded():
+            PIL = get_codec("dec_pillow")
+            if PIL:
+                self._PIL_encodings = PIL.get_encodings()
+            self.jpeg_decoder = get_codec("dec_jpeg")
+            self.webp_decoder = get_codec("dec_webp")
         self.draw_needs_refresh = True
         self.mmap = None
         self.mmap_enabled = False
-        self.jpeg_decoder = get_codec("dec_jpeg")
+
+    def idle_add(self, *_args, **_kwargs):
+        raise NotImplementedError()
 
     def enable_mmap(self, mmap_area):
         self.mmap = mmap_area
@@ -138,15 +159,17 @@ class WindowBackingBase(object):
         #calculate the server CSC modes the server is allowed to use
         #based on the client CSC modes we can convert to in the backing class we use
         #and trim the transparency if we cannot handle it
-        target_rgb_modes = list(rgb_modes)
+        target_rgb_modes = tuple(rgb_modes)
         if not self._alpha_enabled:
-            target_rgb_modes = [x for x in target_rgb_modes if x.find("A")<0]
+            target_rgb_modes = tuple(x for x in target_rgb_modes if x.find("A")<0)
         full_csc_modes = getVideoHelper().get_server_full_csc_modes_for_rgb(*target_rgb_modes)
+        full_csc_modes["webp"] = [x for x in rgb_modes if x in ("BGRX", "BGRA", "RGBX", "RGBA")]
         log("_get_full_csc_modes(%s)=%s (target_rgb_modes=%s)", rgb_modes, full_csc_modes, target_rgb_modes)
         return full_csc_modes
 
 
     def unpremultiply(self, img_data):
+        from xpra.codecs.argb.argb import unpremultiply_argb, unpremultiply_argb_in_place   #@UnresolvedImport
         if type(img_data) not in (str, _buffer):
             try:
                 unpremultiply_argb_in_place(img_data)
@@ -156,6 +179,10 @@ class WindowBackingBase(object):
         return unpremultiply_argb(img_data)
 
 
+    def set_cursor_data(self, cursor_data):
+        self.cursor_data = cursor_data
+
+
     def process_delta(self, raw_data, width, height, rowstride, options):
         """
             Can be called from any thread, decompresses and xors the rgb raw_data,
@@ -169,20 +196,22 @@ class WindowBackingBase(object):
                 assert len(comp)==1, "more than one compressor specified: %s" % str(comp)
                 img_data = compression.decompress_by_name(raw_data, algo=comp[0])
         if len(img_data)!=rowstride * height:
-            deltalog.error("invalid img data length: expected %s but got %s (%s: %s)", rowstride * height, len(img_data), type(img_data), str(img_data)[:256])
+            deltalog.error("Error: invalid img data length: expected %s but got %s (%s: %s)",
+                           rowstride * height, len(img_data), type(img_data), repr_ellipsized(img_data))
             raise Exception("expected %s bytes for %sx%s with rowstride=%s but received %s (%s compressed)" %
                                 (rowstride * height, width, height, rowstride, len(img_data), len(raw_data)))
-        delta = options.intget("delta", -1)
-        bucket = options.intget("bucket", 0)
-        rgb_format = options.strget("rgb_format")
+        delta = options.intget(b"delta", -1)
+        bucket = options.intget(b"bucket", 0)
+        rgb_format = options.strget(b"rgb_format")
         rgb_data = img_data
         if delta>=0:
-            assert bucket>=0 and bucket<DELTA_BUCKETS, "invalid delta bucket number: %s" % bucket
+            assert 0<=bucket<DELTA_BUCKETS, "invalid delta bucket number: %s" % bucket
             if self._delta_pixel_data[bucket] is None:
                 raise Exception("delta region bucket %s references pixmap data we do not have!" % bucket)
             lwidth, lheight, lrgb_format, seq, ldata = self._delta_pixel_data[bucket]
             assert width==lwidth and height==lheight and delta==seq, \
-                "delta bucket %s data does not match: expected %s but got %s" % (bucket, (width, height, delta), (lwidth, lheight, seq))
+                "delta bucket %s data does not match: expected %s but got %s" % (
+                    bucket, (width, height, delta), (lwidth, lheight, seq))
             assert lrgb_format==rgb_format, "delta region uses %s format, was expecting %s" % (rgb_format, lrgb_format)
             deltalog("delta: xoring with bucket %i", bucket)
             rgb_data = xor_str(img_data, ldata)
@@ -194,15 +223,22 @@ class WindowBackingBase(object):
         return rgb_data
 
 
+    def paint_jpeg(self, img_data, x, y, width, height, options, callbacks):
+        img = self.jpeg_decoder.decompress_to_rgb("RGBX", img_data, width, height, options)
+        rgb_format = img.get_pixel_format()
+        img_data = img.get_pixels()
+        rowstride = img.get_rowstride()
+        w = img.get_width()
+        h = img.get_height()
+        self.idle_add(self.paint_rgb, rgb_format, img_data, x, y, w, h, rowstride, options, callbacks)
+
+
     def paint_image(self, coding, img_data, x, y, width, height, options, callbacks):
         """ can be called from any thread """
-        #log("paint_image(%s, %s bytes, %s, %s, %s, %s, %s, %s)", coding, len(img_data), x, y, width, height, options, callbacks)
-        PIL = get_codec("PIL")
-        assert PIL.Image, "PIL.Image not found"
         buf = BytesIOClass(img_data)
-        img = PIL.Image.open(buf)
-        assert img.mode in ("L", "P", "RGB", "RGBA"), "invalid image mode: %s" % img.mode
-        transparency = options.get("transparency", -1)
+        img = Image.open(buf)
+        assert img.mode in ("L", "P", "RGB", "RGBA", "RGBX"), "invalid image mode: %s" % img.mode
+        transparency = options.intget("transparency", -1)
         if img.mode=="P":
             if transparency>=0:
                 #this deals with alpha without any extra work
@@ -216,13 +252,13 @@ class WindowBackingBase(object):
                     if a!=transparency:
                         return 255
                     return 0
-                mask = PIL.Image.eval(img, mask_value)
+                mask = Image.eval(img, mask_value)
                 mask = mask.convert("L")
                 def nomask_value(a):
                     if a!=transparency:
                         return a
                     return 0
-                img = PIL.Image.eval(img, nomask_value)
+                img = Image.eval(img, nomask_value)
                 img = img.convert("RGBA")
                 img.putalpha(mask)
             else:
@@ -230,19 +266,70 @@ class WindowBackingBase(object):
 
         raw_data = img.tobytes("raw", img.mode)
         paint_options = typedict(options)
-        rgb_format = img.mode
-        if rgb_format=="RGB":
+        if img.mode=="RGB":
             #PIL flattens the data to a continuous straightforward RGB format:
             rowstride = width*3
+            rgb_format = options.strget("rgb_format", "")
+            rgb_format = rgb_format.replace("A", "").replace("X", "")
+            #the webp encoder only takes BGRX input,
+            #so we have to swap things around if it was fed "RGB":
+            if rgb_format=="RGB":
+                rgb_format = "BGR"
+            else:
+                rgb_format = "RGB"
             img_data = self.process_delta(raw_data, width, height, rowstride, options)
-        elif rgb_format=="RGBA":
+        elif img.mode in ("RGBA", "RGBX"):
             rowstride = width*4
+            rgb_format = options.strget("rgb_format", img.mode)
+            if coding=="webp":
+                #the webp encoder only takes BGRX input,
+                #so we have to swap things around if it was fed "RGBA":
+                if rgb_format=="RGBA":
+                    rgb_format = "BGRA"
+                elif rgb_format=="RGBX":
+                    rgb_format = "BGRX"
+                elif rgb_format=="BGRA":
+                    rgb_format = "RGBA"
+                elif rgb_format=="BGRX":
+                    rgb_format = "RGBX"
+                else:
+                    log.warn("Warning: unexpected RGB format '%s'", rgb_format)
             img_data = self.process_delta(raw_data, width, height, rowstride, options)
         else:
             raise Exception("invalid image mode: %s" % img.mode)
         self.idle_add(self.do_paint_rgb, rgb_format, img_data, x, y, width, height, rowstride, paint_options, callbacks)
         return False
 
+    def paint_webp(self, img_data, x, y, width, height, options, callbacks):
+        if not self.webp_decoder or WEBP_PILLOW:
+            #if webp is enabled, then Pillow should be able to take care of it:
+            return self.paint_image("webp", img_data, x, y, width, height, options, callbacks)
+        rgb_format = options.strget("rgb_format")
+        has_alpha = options.boolget("has_alpha", False)
+        (
+            buffer_wrapper,
+            width, height, stride, has_alpha,
+            rgb_format,
+            ) = self.webp_decoder.decompress(img_data, has_alpha, rgb_format, self.RGB_MODES)
+        def free_buffer(*_args):
+            buffer_wrapper.free()
+        callbacks.append(free_buffer)
+        data = buffer_wrapper.get_pixels()
+        #if the backing can't handle this format,
+        #ie: tray only supports RGBA
+        if rgb_format not in self.RGB_MODES:
+            from xpra.codecs.rgb_transform import rgb_reformat
+            from xpra.codecs.image_wrapper import ImageWrapper
+            img = ImageWrapper(x, y, width, height, data, rgb_format,
+                               len(rgb_format)*8, stride, len(rgb_format), ImageWrapper.PACKED, True, None)
+            rgb_reformat(img, self.RGB_MODES, has_alpha and self._alpha_enabled)
+            rgb_format = img.get_pixel_format()
+            data = img.get_pixels()
+            stride = img.get_rowstride()
+        #replace with the actual rgb format we get from the decoder:
+        options[b"rgb_format"] = rgb_format
+        return self.paint_rgb(rgb_format, data, x, y, width, height, stride, options, callbacks)
+
     def paint_rgb(self, rgb_format, raw_data, x, y, width, height, rowstride, options, callbacks):
         """ can be called from a non-UI thread
             this method calls process_delta
@@ -257,20 +344,18 @@ class WindowBackingBase(object):
             the actual paint code is in _do_paint_rgb[24|32]
         """
         try:
-            if not options.get("paint", True):
+            if not options.boolget("paint", True):
                 fire_paint_callbacks(callbacks)
                 return
             if self._backing is None:
                 fire_paint_callbacks(callbacks, -1, "no backing")
                 return
             bpp = len(rgb_format)*8
-            assert bpp in (24, 32), "invalid rgb format %s" % rgb_format
+            assert bpp in (24, 32), "invalid rgb format '%s'" % rgb_format
             paint_fn = getattr(self, "_do_paint_rgb%i" % bpp)
-            options["rgb_format"] = rgb_format
+            options[b"rgb_format"] = rgb_format
             success = paint_fn(img_data, x, y, width, height, rowstride, options)
             fire_paint_callbacks(callbacks, success)
-        except KeyboardInterrupt:
-            raise
         except Exception as e:
             if not self._backing:
                 fire_paint_callbacks(callbacks, -1, "paint error on closed backing ignored")
@@ -286,14 +371,22 @@ class WindowBackingBase(object):
         raise Exception("override me!")
 
 
+    def eos(self):
+        dl = self._decoder_lock
+        with dl:
+            self.do_clean_csc_decoder()
+            self.do_clean_video_decoder()
+
+
     def make_csc(self, src_width, src_height, src_format,
                        dst_width, dst_height, dst_format_options, speed):
         global CSC_OPTIONS
         in_options = CSC_OPTIONS.get(src_format, {})
-        assert len(in_options)>0, "no csc options for '%s' input in %s" % (src_format, CSC_OPTIONS)
+        assert in_options, "no csc options for '%s' input in %s" % (src_format, CSC_OPTIONS)
         for dst_format in dst_format_options:
             specs = in_options.get(dst_format)
-            log("make_csc%s specs=%s", (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), specs)
+            log("make_csc%s specs=%s",
+                (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), specs)
             if not specs:
                 continue
             for spec in specs:
@@ -306,7 +399,9 @@ class WindowBackingBase(object):
                                dst_width, dst_height, dst_format, speed)
                     return csc
                 except Exception as e:
-                    log("make_csc%s", (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed), exc_info=True)
+                    log("make_csc%s",
+                        (src_width, src_height, src_format, dst_width, dst_height, dst_format_options, speed),
+                        exc_info=True)
                     log.error("Error: failed to create csc instance %s", spec.codec_class)
                     log.error(" for %s to %s: %s", src_format, dst_format, e)
         log.error("Error: no matching CSC module found")
@@ -324,31 +419,32 @@ class WindowBackingBase(object):
                 v = self.validate_csc_size(spec, src_width, src_height, dst_width, dst_height)
                 if v:
                     log.error("       "+v[0], *v[1:])
-        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s)" % (src_format, src_width, src_height, " or ".join(dst_format_options), dst_width, dst_height, CSC_OPTIONS))
+        raise Exception("no csc module found for %s(%sx%s) to %s(%sx%s) in %s" %
+                        (src_format, src_width, src_height, " or ".join(dst_format_options),
+                         dst_width, dst_height, CSC_OPTIONS))
 
     def validate_csc_size(self, spec, src_width, src_height, dst_width, dst_height):
         if not spec.can_scale and (src_width!=dst_width or src_height!=dst_height):
             return "scaling not suported"
-        elif src_width<spec.min_w:
+        if src_width<spec.min_w:
             return "source width %i is out of range: minimum is %i", src_width, spec.min_w
-        elif src_height<spec.min_h:
+        if src_height<spec.min_h:
             return "source height %i is out of range: minimum is %i", src_height, spec.min_h
-        elif dst_width<spec.min_w:
+        if dst_width<spec.min_w:
             return "target width %i is out of range: minimum is %i", dst_width, spec.min_w
-        elif dst_height<spec.min_h:
+        if dst_height<spec.min_h:
             return "target height %i is out of range: minimum is %i", dst_height, spec.min_h
-        elif src_width>spec.max_w:
+        if src_width>spec.max_w:
             return "source width %i is out of range: maximum is %i", src_width, spec.max_w
-        elif src_height>spec.max_h:
+        if src_height>spec.max_h:
             return "source height %i is out of range: maximum is %i", src_height, spec.max_h
-        elif dst_width>spec.max_w:
+        if dst_width>spec.max_w:
             return "target width %i is out of range: maximum is %i", dst_width, spec.max_w
-        elif dst_height>spec.max_h:
+        if dst_height>spec.max_h:
             return "target height %i is out of range: maximum is %i", dst_height, spec.max_h
         return None
 
     def paint_with_video_decoder(self, decoder_module, coding, img_data, x, y, width, height, options, callbacks):
-        #log("paint_with_video_decoder%s", (decoder_module, coding, "%s bytes" % len(img_data), x, y, width, height, options, callbacks))
         assert decoder_module, "decoder module not found for %s" % coding
         dl = self._decoder_lock
         if dl is None:
@@ -373,21 +469,24 @@ class WindowBackingBase(object):
 
             vd = self._video_decoder
             if vd:
-                if options.get("frame", -1)==0:
+                if options.intget("frame", -1)==0:
                     log("paint_with_video_decoder: first frame of new stream")
                     self.do_clean_video_decoder()
                 elif vd.get_encoding()!=coding:
                     log("paint_with_video_decoder: encoding changed from %s to %s", vd.get_encoding(), coding)
                     self.do_clean_video_decoder()
                 elif vd.get_width()!=enc_width or vd.get_height()!=enc_height:
-                    log("paint_with_video_decoder: video dimensions have changed from %s to %s", (vd.get_width(), vd.get_height()), (enc_width, enc_height))
+                    log("paint_with_video_decoder: video dimensions have changed from %s to %s",
+                        (vd.get_width(), vd.get_height()), (enc_width, enc_height))
                     self.do_clean_video_decoder()
                 elif vd.get_colorspace()!=input_colorspace:
                     #this should only happen on encoder restart, which means this should be the first frame:
-                    log.warn("Warning: colorspace unexpectedly changed from %s to %s", vd.get_colorspace(), input_colorspace)
+                    log.warn("Warning: colorspace unexpectedly changed from %s to %s",
+                             vd.get_colorspace(), input_colorspace)
                     self.do_clean_video_decoder()
             if self._video_decoder is None:
-                log("paint_with_video_decoder: new %s(%s,%s,%s)", decoder_module.Decoder, width, height, input_colorspace)
+                log("paint_with_video_decoder: new %s(%s,%s,%s)",
+                    decoder_module.Decoder, width, height, input_colorspace)
                 vd = decoder_module.Decoder()
                 vd.init_context(coding, enc_width, enc_height, input_colorspace)
                 self._video_decoder = vd
@@ -395,12 +494,14 @@ class WindowBackingBase(object):
 
             img = vd.decompress_image(img_data, options)
             if not img:
-                if options.get("delayed", 0)>0:
+                if options.intget("delayed", 0)>0:
                     #there are further frames queued up,
                     #and this frame references those, so assume all is well:
                     fire_paint_callbacks(callbacks)
                 else:
-                    fire_paint_callbacks(callbacks, False, "video decoder %s failed to decode %i bytes of %s data" % (vd.get_type(), len(img_data), coding))
+                    fire_paint_callbacks(callbacks, False,
+                                         "video decoder %s failed to decode %i bytes of %s data" % (
+                                             vd.get_type(), len(img_data), coding))
                     log.error("Error: decode failed on %s bytes of %s data", len(img_data), coding)
                     log.error(" %sx%s pixels using %s", width, height, vd.get_type())
                     log.error(" frame options:")
@@ -464,11 +565,11 @@ class WindowBackingBase(object):
             see _mmap_send() in server.py for details """
         assert self.mmap_enabled
         data = mmap_read(self.mmap, *img_data)
-        rgb_format = options.strget("rgb_format", "RGB")
+        rgb_format = options.strget(b"rgb_format", b"RGB")
         #Note: BGR(A) is only handled by gl_window_backing
         self.do_paint_rgb(rgb_format, data, x, y, width, height, rowstride, options, callbacks)
 
-    def paint_scroll(self, *args):
+    def paint_scroll(self, _img_data, _options, callbacks):
         raise NotImplementedError("no paint scroll on %s" % type(self))
 
 
@@ -476,43 +577,51 @@ class WindowBackingBase(object):
         """ dispatches the paint to one of the paint_XXXX methods """
         try:
             assert self._backing is not None
-            log("draw_region(%s, %s, %s, %s, %s, %s bytes, %s, %s, %s)", x, y, width, height, coding, len(img_data), rowstride, options, callbacks)
+            log("draw_region(%s, %s, %s, %s, %s, %s bytes, %s, %s, %s)",
+                x, y, width, height, coding, len(img_data), rowstride, options, callbacks)
             coding = bytestostr(coding)
             options["encoding"] = coding            #used for choosing the color of the paint box
             if INTEGRITY_HASH:
-                l = options.get("z.len")
+                l = options.intget("z.len")
                 if l:
                     assert l==len(img_data), "compressed pixel data failed length integrity check: expected %i bytes but got %i" % (l, len(img_data))
-                md5 = options.get("z.md5")
-                if md5:
-                    h = hashlib.md5(img_data)
+                try:
+                    chksum = options.get("z.md5")
+                    if chksum:
+                        h = hashlib.md5(img_data)
+                except ValueError:
+                    chksum = options.get("z.sha1")
+                    if chksum:
+                        h = hashlib.sha1(img_data)
+                if h:
                     hd = h.hexdigest()
-                    assert md5==hd, "pixel data failed compressed md5 integrity check: expected %s but got %s" % (md5, hd)
-                deltalog("passed compressed data integrity checks: len=%s, md5=%s (type=%s)", l, md5, type(img_data))
+                    assert chksum==hd, "pixel data failed compressed chksum integrity check: expected %s but got %s" % (chksum, hd)
+                deltalog("passed compressed data integrity checks: len=%s, chksum=%s (type=%s)", l, chksum, type(img_data))
             if coding == "mmap":
                 self.idle_add(self.paint_mmap, img_data, x, y, width, height, rowstride, options, callbacks)
-            elif coding == "rgb24" or coding == "rgb32":
+            elif coding in ("rgb24", "rgb32"):
                 #avoid confusion over how many bytes-per-pixel we may have:
-                rgb_format = options.get("rgb_format")
-                if rgb_format:
-                    Bpp = len(rgb_format)
-                elif coding=="rgb24":
-                    #legacy:
-                    rgb_format = "RGB"
-                else:
-                    #legacy:
-                    rgb_format = "RGBX"
+                rgb_format = options.strget(b"rgb_format")
+                if not rgb_format:
+                    rgb_format = {
+                        "rgb24" : "RGB",
+                        "rgb32" : "RGBX",
+                        }.get(coding)
                 if rowstride==0:
-                    rowstride = width * Bpp
+                    rowstride = width * len(rgb_format)
                 self.paint_rgb(rgb_format, img_data, x, y, width, height, rowstride, options, callbacks)
             elif coding in VIDEO_DECODERS:
-                self.paint_with_video_decoder(VIDEO_DECODERS.get(coding), coding, img_data, x, y, width, height, options, callbacks)
+                self.paint_with_video_decoder(VIDEO_DECODERS.get(coding),
+                                              coding,
+                                              img_data, x, y, width, height, options, callbacks)
             elif self.jpeg_decoder and coding=="jpeg":
                 self.paint_jpeg(img_data, x, y, width, height, options, callbacks)
+            elif coding == "webp":
+                self.paint_webp(img_data, x, y, width, height, options, callbacks)
             elif coding in self._PIL_encodings:
                 self.paint_image(coding, img_data, x, y, width, height, options, callbacks)
             elif coding == "scroll":
-                self.paint_scroll(x, y, width, height, img_data, options, callbacks)
+                self.paint_scroll(img_data, options, callbacks)
             else:
                 self.do_draw_region(x, y, width, height, coding, img_data, rowstride, options, callbacks)
         except Exception:
@@ -521,7 +630,7 @@ class WindowBackingBase(object):
             else:
                 raise
 
-    def do_draw_region(self, x, y, width, height, coding, img_data, rowstride, options, callbacks):
+    def do_draw_region(self, _x, _y, _width, _height, coding, _img_data, _rowstride, _options, callbacks):
         msg = "invalid encoding: '%s'" % coding
         log.error("Error: %s", msg)
         fire_paint_callbacks(callbacks, False, msg)
