@@ -1,24 +1,29 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.gtk_common.gobject_compat import import_gtk, import_glib
-gtk = import_gtk()
-glib = import_glib()
-
 from xpra.util import envbool, csv
 from xpra.gtk_common.gtk_util import scaled_image
 from xpra.gtk_common.about import about
-from xpra.client.gtk_base.gtk_tray_menu_base import GTKTrayMenuBase, populate_encodingsmenu, \
-            CLIPBOARD_LABEL_TO_NAME, CLIPBOARD_NAME_TO_LABEL, CLIPBOARD_LABELS, CLIPBOARD_DIRECTION_LABELS, CLIPBOARD_DIRECTION_NAME_TO_LABEL, SHOW_UPLOAD
+from xpra.client.gtk_base.gtk_tray_menu_base import (
+    GTKTrayMenuBase, populate_encodingsmenu,
+    CLIPBOARD_LABEL_TO_NAME, CLIPBOARD_NAME_TO_LABEL, CLIPBOARD_LABELS,
+    CLIPBOARD_DIRECTION_LABELS, CLIPBOARD_DIRECTION_NAME_TO_LABEL,
+    SHOW_UPLOAD, SHOW_VERSION_CHECK, RUNCOMMAND_MENU, SHOW_SERVER_COMMANDS, SHOW_SHUTDOWN,
+    )
 from xpra.platform.paths import get_icon
 from xpra.platform.darwin.gui import get_OSXApplication
-
+from xpra.client import mixin_features
+from xpra.gtk_common.gobject_compat import import_gtk, import_glib
 from xpra.log import Logger
+
 log = Logger("osx", "tray", "menu")
 clipboardlog = Logger("osx", "menu", "clipboard")
 
+gtk = import_gtk()
+glib = import_glib()
+
 
 #control which menus are shown in the OSX global menu:
 SHOW_FEATURES_MENU = True
@@ -27,6 +32,7 @@ SHOW_ENCODINGS_MENU = True
 SHOW_ACTIONS_MENU = True
 SHOW_INFO_MENU = True
 SHOW_CLIPBOARD_MENU = True
+SHOW_SERVER_MENU = True
 
 SHOW_ABOUT_XPRA = True
 
@@ -59,7 +65,6 @@ class OSXMenuHelper(GTKTrayMenuBase):
         log("OSXMenuHelper(%s)", client)
         self.menu_bar = None
         self.hidden_window = None
-        self.keyboard = None
         self.menus = {}             #all the "top-level" menu items we manage
         self.app_menus = {}         #the ones added to the app_menu via insert_app_menu_item (which cannot be removed!)
         self.full = False
@@ -68,8 +73,6 @@ class OSXMenuHelper(GTKTrayMenuBase):
 
     def set_client(self, client):
         self.client = client
-        if client and client.keyboard_helper:
-            self.keyboard = client.keyboard_helper.keyboard
         #if we call add_about before the main loop is ready,
         #things don't work...
         if client and SHOW_ABOUT_XPRA:
@@ -150,8 +153,7 @@ class OSXMenuHelper(GTKTrayMenuBase):
     def get_menu(self, label):
         if SINGLE_MENU:
             return self.app_menus.get(label)
-        else:
-            return self.menus.get(label)
+        return self.menus.get(label)
 
 
     def add_about(self):
@@ -179,19 +181,20 @@ class OSXMenuHelper(GTKTrayMenuBase):
         if SHOW_INFO_MENU:
             info_menu = self.make_menu()
             info_menu.append(self.make_sessioninfomenuitem())
+            if SHOW_VERSION_CHECK:
+                info_menu.append(self.make_updatecheckmenuitem())
             info_menu.append(self.make_bugreportmenuitem())
             menus.append(("Info", info_menu))
         if SHOW_FEATURES_MENU:
             features_menu = self.make_menu()
             menus.append(("Features", features_menu))
-            features_menu.add(self.make_bellmenuitem())
-            features_menu.add(self.make_cursorsmenuitem())
-            features_menu.add(self.make_notificationsmenuitem())
-            features_menu.add(self.make_swapkeysmenuitem())
-            features_menu.add(self.make_numlockmenuitem())
-            features_menu.add(self.make_openglmenuitem())
-            features_menu.add(self.make_scalingmenuitem())
-        if SHOW_CLIPBOARD_MENU:
+            self.append_featuresmenuitems(features_menu)
+            if mixin_features.windows:
+                features_menu.add(self.make_swapkeysmenuitem())
+                features_menu.add(self.make_invertmousewheelmenuitem())
+                features_menu.add(self.make_numlockmenuitem())
+                features_menu.add(self.make_scalingmenuitem())
+        if mixin_features.clipboard and SHOW_CLIPBOARD_MENU:
             clipboard_menu = self.make_menu()
             menus.append(("Clipboard", clipboard_menu))
             for label in CLIPBOARD_LABELS:
@@ -201,17 +204,17 @@ class OSXMenuHelper(GTKTrayMenuBase):
                 clipboard_menu.add(self.make_clipboard_submenuitem(label, self._clipboard_direction_changed))
             clipboard_menu.show_all()
             self.client.after_handshake(self.set_clipboard_menu, clipboard_menu)
-        if SHOW_SOUND_MENU:
+        if mixin_features.audio and SHOW_SOUND_MENU:
             sound_menu = self.make_menu()
-            if self.client.speaker_allowed and len(self.client.speaker_codecs)>0:
+            if self.client.speaker_allowed and self.client.speaker_codecs:
                 sound_menu.add(self.make_speakermenuitem())
-            if self.client.microphone_allowed and len(self.client.microphone_codecs)>0:
+            if self.client.microphone_allowed and self.client.microphone_codecs:
                 sound_menu.add(self.make_microphonemenuitem())
             menus.append(("Sound", sound_menu))
-        if SHOW_ENCODINGS_MENU:
+        if mixin_features.windows and SHOW_ENCODINGS_MENU:
             encodings_menu = self.make_menu()
-            def set_encodings_menu(*args):
-                from xpra.codecs.loader import PREFERED_ENCODING_ORDER
+            def set_encodings_menu(*_args):
+                from xpra.codecs.codec_constants import PREFERED_ENCODING_ORDER
                 server_encodings = list(self.client.server_encodings)
                 encodings = [x for x in PREFERED_ENCODING_ORDER if x in self.client.get_encodings()]
                 if self.client.server_auto_video_encoding:
@@ -220,30 +223,39 @@ class OSXMenuHelper(GTKTrayMenuBase):
                 populate_encodingsmenu(encodings_menu, self.get_current_encoding, self.set_current_encoding, encodings, server_encodings)
             self.client.after_handshake(set_encodings_menu)
             menus.append(("Encoding", encodings_menu))
-        if SHOW_ACTIONS_MENU:
+        if mixin_features.windows and SHOW_ACTIONS_MENU:
             actions_menu = self.make_menu()
             actions_menu.add(self.make_refreshmenuitem())
             actions_menu.add(self.make_raisewindowsmenuitem())
+            menus.append(("Actions", actions_menu))
+        if RUNCOMMAND_MENU or SHOW_SERVER_COMMANDS or SHOW_UPLOAD or SHOW_SHUTDOWN:
+            server_menu = self.make_menu()
+            if SHOW_SHUTDOWN:
+                server_menu.append(self.make_shutdownmenuitem())
             #set_sensitive(bool) does not work on OSX,
             #so we only add the menu item if it does something
-            def add_ah(*args):
-                if self.client.start_new_commands:
-                    actions_menu.add(self.make_runcommandmenu())
+            def add_ah(*_args):
+                if self.client.server_start_new_commands:
+                    server_menu.add(self.make_runcommandmenuitem())
+                if SHOW_SERVER_COMMANDS and self.client.server_commands_info:
+                    server_menu.append(self.make_servercommandsmenuitem())
                 if SHOW_UPLOAD and self.client.remote_file_transfer:
-                    actions_menu.add(self.make_uploadmenuitem())
+                    server_menu.add(self.make_uploadmenuitem())
             self.client.after_handshake(add_ah)
-            menus.append(("Actions", actions_menu))
+            menus.append(("Server", server_menu))
         menus.append((SEPARATOR+"-EXTRAS", None))
         return menus
 
 
     def _clipboard_direction_changed(self, item, label):
-        clipboardlog("_clipboard_direction_changed(%s, %s) clipboard_change_pending=%s", item, label, self._clipboard_change_pending)
+        clipboardlog("_clipboard_direction_changed(%s, %s) clipboard_change_pending=%s",
+                     item, label, self._clipboard_change_pending)
         label = self.select_clipboard_menu_option(item, label, CLIPBOARD_DIRECTION_LABELS)
         self.do_clipboard_direction_changed(label or "")
 
     def _remote_clipboard_changed(self, item, label):
-        clipboardlog("_remote_clipboard_changed(%s, %s) clipboard_change_pending=%s", item, label, self._clipboard_change_pending)
+        clipboardlog("_remote_clipboard_changed(%s, %s) clipboard_change_pending=%s",
+                     item, label, self._clipboard_change_pending)
         #ensure this is the only clipboard label selected:
         label = self.select_clipboard_menu_option(item, label, CLIPBOARD_LABELS)
         if not label:
@@ -264,7 +276,8 @@ class OSXMenuHelper(GTKTrayMenuBase):
         clipboard_item = self.checkitem(label)
         clipboard_item.set_draw_as_radio(True)
         def clipboard_option_changed(item):
-            clipboardlog("clipboard_option_changed(%s) label=%s, callback=%s clipboard_change_pending=%s", item, label, cb, self._clipboard_change_pending)
+            clipboardlog("clipboard_option_changed(%s) label=%s, callback=%s clipboard_change_pending=%s",
+                         item, label, cb, self._clipboard_change_pending)
             if cb:
                 cb(item, label)
         clipboard_item.connect("toggled", clipboard_option_changed)
@@ -275,7 +288,8 @@ class OSXMenuHelper(GTKTrayMenuBase):
         #(can be specified as a menuitem object, or using its label)
         #all the other menu items whose labels are specified will be made inactive
         #(we use a flag to prevent reentry)
-        clipboardlog("select_clipboard_menu_option(%s, %s, %s) clipboard_change_pending=%s", item, label, labels, self._clipboard_change_pending)
+        clipboardlog("select_clipboard_menu_option(%s, %s, %s) clipboard_change_pending=%s",
+                     item, label, labels, self._clipboard_change_pending)
         if self._clipboard_change_pending:
             return None
         clipboard = self.get_menu("Clipboard")
@@ -286,8 +300,8 @@ class OSXMenuHelper(GTKTrayMenuBase):
         selected_items = [x for x in all_items if x==item] + [x for x in all_items if x.get_label()==label]
         if not selected_items:
             log.error("Error: cannot find any clipboard menu options to match '%s'", label)
-            log.error(" all menu items: %s", csv(all_items))
-            log.error(" selected: %s", csv(selected_items))
+            log.error(" all menu items: %s", csv(x.get_label() for x in all_items))
+            log.error(" selected: %s", csv(x.get_label() for x in selected_items))
             return None
         self._clipboard_change_pending = True
         sel = selected_items[0]
@@ -300,14 +314,14 @@ class OSXMenuHelper(GTKTrayMenuBase):
         self._clipboard_change_pending = False
         return label
 
-    def set_clipboard_menu(self, clipboard_menu):
+    def set_clipboard_menu(self, _clipboard_menu):
         #find the menu item matching the current settings,
         #and select it
         try:
             label = CLIPBOARD_NAME_TO_LABEL.get(self.client.clipboard_helper.remote_clipboard)
+            self.select_clipboard_menu_option(None, label, CLIPBOARD_LABELS)
         except:
-            label = None
-        self.select_clipboard_menu_option(None, label, CLIPBOARD_LABELS)
+            pass
         direction_label = CLIPBOARD_DIRECTION_NAME_TO_LABEL.get(self.client.client_clipboard_direction, "Disabled")
         clipboardlog("direction(%s)=%s", self.client.client_clipboard_direction, direction_label)
         self.select_clipboard_menu_option(None, direction_label, CLIPBOARD_DIRECTION_LABELS)
@@ -320,35 +334,58 @@ class OSXMenuHelper(GTKTrayMenuBase):
     def set_speedmenu(self, *args):
         pass
 
+    def _get_keyboard(self):
+        if not self.client or not self.client.keyboard_helper:
+            return None
+        return self.client.keyboard_helper.keyboard
+
     def make_swapkeysmenuitem(self):
         def swapkeys_toggled(*args):
             v = self.swapkeys_menuitem.get_active()
-            log("swapkeys_toggled(%s) swap keys enabled=%s", args, v)
-            if self.keyboard:
-                self.keyboard.swap_keys = v
+            keyboard = self._get_keyboard()
+            log("swapkeys_toggled(%s) keyboard=%s, swap keys enabled=%s", args, keyboard, v)
+            if keyboard:
+                keyboard.swap_keys = v
         self.swapkeys_menuitem = self.checkitem("Control/Command Key Swap", swapkeys_toggled)
         def set_swapkeys_menuitem(*args):
-            if self.keyboard:
-                log("set_swapkeys_menuitem(%s) swap_keys=%s", args, self.keyboard.swap_keys)
-                self.swapkeys_menuitem.set_active(self.keyboard.swap_keys)
+            keyboard = self._get_keyboard()
+            if keyboard:
+                log("set_swapkeys_menuitem(%s) keyboard=%s, swap_keys=%s", args, keyboard, keyboard.swap_keys)
+                self.swapkeys_menuitem.set_active(keyboard.swap_keys)
             else:
                 log("set_swapkeys_menuitem(%s) no keyboard!", args)
                 self.swapkeys_menuitem.set_sensitive(False)
         self.client.after_handshake(set_swapkeys_menuitem)
-        return  self.swapkeys_menuitem
+        return self.swapkeys_menuitem
+
+    def make_invertmousewheelmenuitem(self):
+        def invert_toggled(*args):
+            v = self.mousewheel_menuitem.get_active()
+            log("invert_toggled(%s) invert enabled=%s", args, v)
+            if v:
+                self.client.wheel_map[4] = 5
+                self.client.wheel_map[5] = 4
+            else:
+                self.client.wheel_map[4] = 4
+                self.client.wheel_map[5] = 5
+        self.mousewheel_menuitem = self.checkitem("Invert Mouse Wheel", invert_toggled)
+        self.mousewheel_menuitem.set_active(self.client.wheel_map.get(4)!=4)
+        return self.mousewheel_menuitem
 
     def make_numlockmenuitem(self):
         def numlock_toggled(*args):
             v = self.numlock_menuitem.get_active()
+            keyboard = self._get_keyboard()
             log("numlock_toggled(%s) menu active=%s", args, v)
-            if self.keyboard:
-                self.keyboard.num_lock_state = v
+            if keyboard:
+                keyboard.num_lock_state = v
         self.numlock_menuitem = self.checkitem("Num Lock", cb=numlock_toggled)
         self.numlock_menuitem.set_active(True)
         def set_numlock_menuitem(*args):
-            if self.keyboard:
-                log("set_numlock_menuitem(%s) num_lock_state=%s", args, self.keyboard.num_lock_state)
-                self.numlock_menuitem.set_active(self.keyboard.num_lock_state)
+            keyboard = self._get_keyboard()
+            if keyboard:
+                log("set_numlock_menuitem(%s) keyboard=%s, num_lock_state=%s", args, keyboard, keyboard.num_lock_state)
+                self.numlock_menuitem.set_active(keyboard.num_lock_state)
             else:
                 log("set_numlock_menuitem(%s) no keyboard!", args)
                 self.swapkeys_menuitem.set_sensitive(False)
