@@ -1,24 +1,23 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2014-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2014-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 #default implementation using pycups
 import sys
 import os
-import cups
 import time
-import subprocess
+from subprocess import PIPE, Popen
 import shlex
-import urllib
 from threading import Lock
+import cups
 
-from xpra.os_util import OSX
-from xpra.util import engs, envint, envbool
+from xpra.os_util import OSX, PYTHON3, bytestostr
+from xpra.util import engs, envint, envbool, parse_simple_dict
 from xpra.log import Logger
-log = Logger("printing")
 
+log = Logger("printing")
 
 SIMULATE_PRINT_FAILURE = envint("XPRA_SIMULATE_PRINT_FAILURE")
 
@@ -49,30 +48,26 @@ PRINTER_PREFIX = os.environ.get("XPRA_PRINTER_PREFIX", PRINTER_PREFIX)
 DEFAULT_CUPS_DBUS = int(not OSX)
 CUPS_DBUS = envint("XPRA_CUPS_DBUS", DEFAULT_CUPS_DBUS)
 POLLING_DELAY = envint("XPRA_CUPS_POLLING_DELAY", 60)
-log("pycups settings: DEFAULT_CUPS_DBUS=%s, CUPS_DBUS=%s, POLLING_DELAY=%s", DEFAULT_CUPS_DBUS, CUPS_DBUS, POLLING_DELAY)
-log("pycups settings: PRINTER_PREFIX=%s, ADD_LOCAL_PRINTERS=%s", PRINTER_PREFIX, ADD_LOCAL_PRINTERS)
+log("pycups settings: DEFAULT_CUPS_DBUS=%s, CUPS_DBUS=%s, POLLING_DELAY=%s",
+    DEFAULT_CUPS_DBUS, CUPS_DBUS, POLLING_DELAY)
+log("pycups settings: PRINTER_PREFIX=%s, ADD_LOCAL_PRINTERS=%s",
+    PRINTER_PREFIX, ADD_LOCAL_PRINTERS)
 log("pycups settings: FORWARDER_TMPDIR=%s", FORWARDER_TMPDIR)
 log("pycups settings: SKIPPED_PRINTERS=%s", SKIPPED_PRINTERS)
 
-MIMETYPE_TO_PRINTER = {"application/postscript" : "Generic PostScript Printer",
-                       "application/pdf"        : "Generic PDF Printer"}
-MIMETYPE_TO_PPD = {"application/postscript"     : "CUPS-PDF.ppd",
-                   "application/pdf"            : "Generic-PDF_Printer-PDF.ppd"}
+MIMETYPE_TO_PRINTER = {
+    "application/postscript"    : "Generic PostScript Printer",
+    "application/pdf"           : "Generic PDF Printer",
+    }
+MIMETYPE_TO_PPD = {
+    "application/postscript"    : "CUPS-PDF.ppd",
+    "application/pdf"           : "Generic-PDF_Printer-PDF.ppd",
+    }
 
 
-DEFAULT_CUPS_OPTIONS = {}
 dco = os.environ.get("XPRA_DEFAULT_CUPS_OPTIONS", "fit-to-page=True")
-if dco:
-    for opt in dco.split(","):
-        opt = opt.strip(" ")
-        parts = opt.split("=", 1)
-        if len(parts)!=2:
-            log.warn("Warning: invalid cups option: '%s'", opt)
-            continue
-        #is it a boolean?
-        k,v = parts
-        DEFAULT_CUPS_OPTIONS[k] = v
-    log("DEFAULT_CUPS_OPTIONS=%s", DEFAULT_CUPS_OPTIONS)
+DEFAULT_CUPS_OPTIONS = parse_simple_dict(dco)
+log("DEFAULT_CUPS_OPTIONS=%s", DEFAULT_CUPS_OPTIONS)
 
 
 #allows us to inject the lpadmin and lpinfo commands from the config file
@@ -94,27 +89,31 @@ def find_ppd_file(short_name, filename):
     if ev and os.path.exists(ev):
         log("using environment override for %s ppd file: %s", short_name, ev)
         return ev
-    paths = ["/usr/share/cups/model",           #used on Fedora and others
-             "/usr/share/ppd/cups-pdf",         #used on Ubuntu
-             "/usr/share/ppd/cupsfilters",
-             "/usr/local/share/cups/model",     #install from source with /usr/local prefix
-             #if your distro uses something else, please file a bug so we can add it
-            ]
+    paths = []
+    for p in os.environ.get("XDG_DATA_DIRS", "/usr/local/share:/usr/share").split(":"):
+        if os.path.exists(p) and os.path.isdir(p):
+            paths.append(os.path.join(p, "cups", "model"))      #used on Fedora and others
+            paths.append(os.path.join(p, "ppd", "cups-pdf"))    #used on Fedora and others
+            paths.append(os.path.join(p, "ppd", "cupsfilters"))
+    log("find ppd file: paths=%s", paths)
     for p in paths:
         f = os.path.join(p, filename)
-        if os.path.exists(f):
+        if os.path.exists(f) and os.path.isfile(f):
             return f
     log("cannot find %s in %s", filename, paths)
     return None
 
 
 def get_lpinfo_drv(make_and_model):
+    if not LPINFO:
+        log.error("Error: lpinfo command is not defined")
+        return None
     command = shlex.split(LPINFO)+["--make-and-model", make_and_model, "-m"]
     def preexec():
         os.setsid()
     log("get_lpinfo_drv(%s) command=%s", make_and_model, command)
     try:
-        proc = subprocess.Popen(command, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False, close_fds=True, preexec_fn=preexec)
+        proc = Popen(command, stdout=PIPE, stderr=PIPE, close_fds=True, preexec_fn=preexec)
     except Exception as e:
         log("get_lp_info_drv(%s) lpinfo command %s failed", make_and_model, command, exc_info=True)
         log.error("Error: lpinfo command failed to run")
@@ -149,7 +148,7 @@ def get_lpinfo_drv(make_and_model):
         log.warn("Warning: lpinfo command failed and returned %s", proc.returncode)
         log.warn(" command used: '%s'", " ".join(command))
         return None
-    if sys.version_info[0]>=3:
+    if PYTHON3:
         try:
             out = out.decode()
         except:
@@ -246,11 +245,11 @@ def exec_lpadmin(args, success_cb=None):
     def preexec():
         os.setsid()
     log("exec_lpadmin(%s) command=%s", args, command)
-    proc = subprocess.Popen(command, stdin=None, stdout=None, stderr=None, shell=False, close_fds=True, preexec_fn=preexec)
+    proc = Popen(command, close_fds=True, preexec_fn=preexec)
     #use the global child reaper to make sure this doesn't end up as a zombie
     from xpra.child_reaper import getChildReaper
     cr = getChildReaper()
-    def check_returncode(proc_cb):
+    def check_returncode(_proc_cb):
         returncode = proc.poll()
         log("returncode(%s)=%s", command, returncode)
         if returncode!=0:
@@ -276,7 +275,11 @@ def add_printer(name, options, info, location, attributes={}, success_cb=None):
     log("add_printer%s", (name, options, info, location, attributes, success_cb))
     mimetypes = options.get("mimetypes", [DEFAULT_MIMETYPE])
     if not mimetypes:
-        log.error("Error: no mimetypes specified for printer %s", name)
+        log.error("Error: no mimetypes specified for printer '%s'", name)
+        return
+    xpra_printer_name = PRINTER_PREFIX+sanitize_name(name)
+    if xpra_printer_name in get_all_printers():
+        log.warn("Warning: not adding duplicate printer '%s'", name)
         return
     #find a matching definition:
     mimetype, printer_def = None, None
@@ -293,9 +296,13 @@ def add_printer(name, options, info, location, attributes={}, success_cb=None):
         log.error("Error: cannot add printer '%s':", name)
         log.error(" the printing system does not support %s", " or ".join(mimetypes))
         return
+    if PYTHON3:
+        from urllib.parse import urlencode      #@UnresolvedImport @UnusedImport
+    else:
+        from urllib import urlencode            #@Reimport
     command = [
-               "-p", PRINTER_PREFIX+sanitize_name(name),
-               "-v", FORWARDER_BACKEND+":"+FORWARDER_TMPDIR+"?"+urllib.urlencode(attributes),
+               "-p", xpra_printer_name,
+               "-v", FORWARDER_BACKEND+":"+FORWARDER_TMPDIR+"?"+urlencode(attributes),
                "-D", info,
                "-L", location,
                ]
@@ -418,8 +425,9 @@ def print_files(printer, filenames, title, options):
         raise Exception("invalid printer: '%s'" % printer)
     log("pycups.print_files%s", (printer, filenames, title, options))
     actual_options = DEFAULT_CUPS_OPTIONS.copy()
-    used_options = dict((str(k),str(v)) for k,v in options.items() if str(k) in CUPS_OPTIONS_WHITELIST)
-    unused_options = dict((str(k),str(v)) for k,v in options.items() if str(k) not in CUPS_OPTIONS_WHITELIST)
+    s = bytestostr
+    used_options = dict((s(k),s(v)) for k,v in options.items() if s(k) in CUPS_OPTIONS_WHITELIST)
+    unused_options = dict((s(k),s(v)) for k,v in options.items() if s(k) not in CUPS_OPTIONS_WHITELIST)
     log("used options=%s", used_options)
     log("unused options=%s", unused_options)
     actual_options.update(used_options)
@@ -485,18 +493,12 @@ def get_info():
 
 
 def main():
-    if "-v" in sys.argv or "--verbose" in sys.argv:
-        from xpra.log import add_debug_category, enable_debug_for
-        add_debug_category("printing")
-        enable_debug_for("printing")
-        try:
-            sys.argv.remove("-v")
-        except:
-            pass
-        try:
-            sys.argv.remove("--verbose")
-        except:
-            pass
+    for arg in list(sys.argv):
+        if arg in ("-v", "--verbose"):
+            from xpra.log import add_debug_category, enable_debug_for
+            add_debug_category("printing")
+            enable_debug_for("printing")
+            sys.argv.remove(arg)
 
     from xpra.platform import program_context
     from xpra.log import enable_color
