@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2017 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,12 +10,16 @@ import errno
 import os.path
 import sys
 import ctypes
-
 from ctypes import WINFUNCTYPE, WinDLL, POINTER, byref, c_int
 from ctypes.wintypes import BOOL, HANDLE, DWORD, LPWSTR, LPCWSTR, LPVOID, POINT, WORD, SMALL_RECT
 
+from xpra.util import envbool
+from xpra.os_util import PYTHON2, PYTHON3
 from xpra.platform.win32 import constants as win32con
-from xpra.platform.win32.common import SetConsoleTitleA, GetConsoleScreenBufferInfo, MessageBoxA, GetLastError, kernel32
+from xpra.platform.win32.common import (
+    SetConsoleTitleA, GetConsoleScreenBufferInfo,
+    MessageBoxA, GetLastError, kernel32,
+    )
 
 
 GetStdHandle = WINFUNCTYPE(HANDLE, DWORD)(("GetStdHandle", kernel32))
@@ -34,22 +38,27 @@ GetConsoleCP = kernel32.GetConsoleCP
 
 #redirect output if we're not running from a console:
 frozen = getattr(sys, 'frozen', False)
-REDIRECT_OUTPUT = frozen is True and GetConsoleCP()==0
+REDIRECT_OUTPUT = envbool("XPRA_REDIRECT_OUTPUT", frozen is True and GetConsoleCP()==0)
 if frozen:
     #cx_freeze paths:
-    def jedir(relpathname):
-        return os.path.join(os.path.dirname(sys.executable), relpathname)
-    def addsyspath(relpathname):
-        v = jedir(relpathname)
+    PATH = os.environ.get("PATH", "").split(os.pathsep)
+    edir = os.path.dirname(sys.executable)
+    def jedir(*paths):
+        return os.path.join(*([edir]+list(paths)))
+    def addsyspath(*paths):
+        v = jedir(*paths)
         if os.path.exists(v):
-            sys.path.append(v)
+            if v not in sys.path:
+                sys.path.append(v)
+            if os.path.isdir(v) and v not in PATH:
+                PATH.append(v)
     addsyspath('')
-    addsyspath('bin')
-    addsyspath('bin\\etc')
-    addsyspath('bin\\lib')
-    addsyspath('bin\\share')
-    addsyspath('bin\\library.zip')
-    os.environ['GI_TYPELIB_PATH'] = jedir('lib\\girepository-1.0')
+    addsyspath('lib')
+    os.environ['GI_TYPELIB_PATH'] = jedir('lib', 'girepository-1.0')
+    os.environ["PATH"] = os.pathsep.join(PATH)
+
+#don't know why this breaks with Python 3 yet...
+FIX_UNICODE_OUT = envbool("XPRA_FIX_UNICODE_OUT", not REDIRECT_OUTPUT and PYTHON2)
 
 
 def is_wine():
@@ -58,7 +67,7 @@ def is_wine():
             import _winreg as winreg
         except ImportError:
             import winreg   #@UnresolvedImport @Reimport
-        hKey = winreg.OpenKey(win32con.HKEY_LOCAL_MACHINE, r"Software\\Wine")
+        hKey = winreg.OpenKey(win32con.HKEY_LOCAL_MACHINE, r"Software\\Wine")   #@UndefinedVariable
         return hKey is not None
     except:
         #no wine key, assume not present and wait for input
@@ -72,13 +81,18 @@ def set_prgname(name):
     prg_name = name
     try:
         SetConsoleTitleA(name)
-        import glib
-        glib.set_prgname(name)
+        if PYTHON2:
+            import glib     #@UnresolvedImport
+            glib.set_prgname(name)
     except:
         pass
 
 
 def fix_unicode_out():
+    if PYTHON3:
+        _unicode = str
+    else:
+        _unicode = unicode
     #code found here:
     #http://stackoverflow.com/a/3259271/428751
     import codecs
@@ -89,7 +103,9 @@ def fix_unicode_out():
     # So be paranoid about catching errors and reporting them to original_stderr,
     # so that we can at least see them.
     def _complain(message):
-        print >>original_stderr, message if isinstance(message, str) else repr(message)
+        if not isinstance(message, str):
+            message = repr(message)
+        original_stderr.write("%s\n" % message)
 
     # Work around <http://bugs.python.org/issue6058>.
     codecs.register(lambda name: codecs.lookup('utf-8') if name == 'cp65001' else None)
@@ -167,17 +183,18 @@ def fix_unicode_out():
                         try:
                             self._stream.flush()
                         except Exception as e:
-                            _complain("%s.flush: %r from %r" % (self.name, e, self._stream))
-                            raise
+                            if not self.closed:
+                                _complain("%s.flush: %r from %r" % (self.name, e, self._stream))
+                                raise
 
                 def write(self, text):
                     try:
                         if self._hConsole is None:
-                            if isinstance(text, unicode):
+                            if isinstance(text, _unicode):
                                 text = text.encode('utf-8')
                             self._stream.write(text)
                         else:
-                            if not isinstance(text, unicode):
+                            if not isinstance(text, _unicode):
                                 text = str(text).decode('utf-8')
                             remaining = len(text)
                             while remaining:
@@ -193,16 +210,18 @@ def fix_unicode_out():
                                     break
                                 text = text[n.value:]
                     except Exception as e:
-                        _complain("%s.write: %r" % (self.name, e))
-                        raise
+                        if not self.closed:
+                            _complain("%s.write: %r" % (self.name, e))
+                            raise
 
                 def writelines(self, lines):
                     try:
                         for line in lines:
                             self.write(line)
                     except Exception as e:
-                        _complain("%s.writelines: %r" % (self.name, e))
-                        raise
+                        if not self.closed:
+                            _complain("%s.writelines: %r" % (self.name, e))
+                            raise
 
             if real_stdout:
                 sys.stdout = UnicodeOutput(hStdout, None, STDOUT_FILENO, '<Unicode console stdout>')
@@ -234,13 +253,18 @@ class CONSOLE_SCREEN_BUFFER_INFO(ctypes.Structure):
 _wait_for_input = False
 def set_wait_for_input():
     global _wait_for_input
+    wfi = os.environ.get("XPRA_WAIT_FOR_INPUT")
+    if wfi is not None:
+        _wait_for_input = wfi!="0"
+        return
     if is_wine():
         #don't wait for input when running under wine
         #(which usually does not popup a new shell window)
         _wait_for_input = False
         return
-    if os.environ.get("MSYSCON"):
-        #msys environment doesn't popup a new shell window
+    if os.environ.get("TERM", "")=="xterm":
+        #msys, cygwin and git bash environments don't popup a new shell window
+        #and they all set TERM=xterm
         _wait_for_input = False
         return
     try:
@@ -270,20 +294,20 @@ def set_wait_for_input():
                 print("error accessing console: %s" % e)
 
 def do_init():
+    if FIX_UNICODE_OUT:
+        fix_unicode_out()
     if not REDIRECT_OUTPUT:
-        if sys.version_info[0]<3:
-            #don't know why this breaks with Python 3 yet...
-            fix_unicode_out()
         #figure out if we want to wait for input at the end:
         set_wait_for_input()
         return
-    from xpra.platform import get_prgname
-    LOG_FILENAME = (get_prgname() or "Xpra")+".log"
-    from paths import _get_data_dir
-    d = _get_data_dir()
-    log_file = os.path.join(d, LOG_FILENAME)
-    sys.stdout = open(log_file, "a")
-    sys.stderr = sys.stdout
+    if envbool("XPRA_LOG_TO_FILE", True):
+        from xpra.platform import get_prgname
+        LOG_FILENAME = (get_prgname() or "Xpra")+".log"
+        from xpra.platform.win32.paths import _get_data_dir
+        d = _get_data_dir()
+        log_file = os.path.join(d, LOG_FILENAME)
+        sys.stdout = open(log_file, "a")
+        sys.stderr = sys.stdout
 
 
 MB_ICONEXCLAMATION  = 0x00000030
@@ -292,16 +316,14 @@ MB_SYSTEMMODAL      = 0x00001000
 def _show_message(message, uType):
     global prg_name
     #TODO: detect cx_freeze equivallent
-    GUI_MODE = hasattr(sys, "frozen") and sys.frozen=="windows_exe"
-    from xpra.util import envbool
-    SHOW_MESSAGEBOX = envbool("XPRA_MESSAGEBOX", True)
-    if SHOW_MESSAGEBOX and GUI_MODE:
+    SHOW_MESSAGEBOX = envbool("XPRA_MESSAGEBOX", REDIRECT_OUTPUT)
+    if SHOW_MESSAGEBOX:
         #try to use an alert box since no console output will be shown:
         try:
-            MessageBoxA(0, message, prg_name, uType)
+            MessageBoxA(0, message.encode(), prg_name.encode(), uType)
             return
-        except:
-            pass
+        except Exception as e:
+            print("Error: cannot show alert box: %s" % (e,))
     print(message)
 
 def command_info(message):
@@ -311,31 +333,16 @@ def command_error(message):
     _show_message(message, MB_ICONEXCLAMATION | MB_SYSTEMMODAL)
 
 
-def get_main_fallback():
-    set_wait_for_input()
-    global _wait_for_input
-    if _wait_for_input:
-        #something will be shown in a console,
-        #so don't bother showing anything else
-        return None
-    #try the launcher (better than nothing!)
-    try:
-        from xpra.client.gtk_base.client_launcher import main
-        return main
-    except:
-        return None
-
-
 def do_clean():
     global _wait_for_input
     if _wait_for_input:
         print("\nPress Enter to close")
         try:
             sys.stdout.flush()
-        except:
+        except IOError:
             pass
         try:
             sys.stderr.flush()
-        except:
+        except IOError:
             pass
         sys.stdin.readline()
