@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,6 +8,8 @@ import os.path
 import sys
 import ctypes
 
+from xpra.os_util import get_util_logger, PYTHON3
+
 shell32 = ctypes.WinDLL("shell32", use_last_error=True)
 SHGetFolderPath = shell32.SHGetFolderPathW
 
@@ -40,22 +42,58 @@ def _get_data_dir():
     return data_dir
 
 
-def do_get_icon_dir():
+def do_get_resources_dir():
     from xpra.platform.paths import get_app_dir
-    return os.path.join(get_app_dir(), "icons")
+    app_dir = get_app_dir()
+    prefix = os.environ.get("MINGW_PREFIX")
+    for d in (app_dir, prefix):
+        if not d or not os.path.isdir(d):
+            continue
+        share_xpra = os.path.join(d, "share", "xpra")
+        if os.path.exists(share_xpra):
+            return share_xpra
+    return app_dir
+
+def do_get_icon_dir():
+    from xpra.platform.paths import get_resources_dir
+    return os.path.join(get_resources_dir(), "icons")
 
 
-def do_get_system_conf_dirs():
-    #ie: "C:\Documents and Settings\All Users\Application Data\Xpra" with XP
-    #or: "C:\ProgramData\Xpra" with Vista onwards
+def get_program_data_dir():
+    #ie: "C:\ProgramData"
     try:
         buf = ctypes.create_unicode_buffer(ctypes.wintypes.MAX_PATH)
         SHGetFolderPath(0, CSIDL_COMMON_APPDATA, None, 0, buf)
         if buf.value:
-            return [os.path.join(buf.value, "Xpra")]
+            return buf.value
     except:
-        pass
-    return []
+        get_util_logger().debug("get_program_data_dir()", exc_info=True)
+    return u"C:\\ProgramData"
+
+def do_get_system_conf_dirs():
+    #ie: C:\ProgramData\Xpra
+    return [os.path.join(get_program_data_dir(), u"Xpra")]
+
+def do_get_ssh_conf_dirs():
+    from xpra.scripts.config import python_platform
+    if python_platform.architecture()[0]=="32bit":
+        system32 = "SysNative"
+    else:
+        system32 = "System32"
+    windows_dir = os.environ.get("SystemRoot", os.environ.get("WINDIR", "C:\\Windows"))
+    openssh_dir = os.path.join(windows_dir, system32, "OpenSSH")
+    return [
+        os.path.join(get_program_data_dir(), "ssh"),    #ie: C:\ProgramData\ssh
+        "%APPDATA%\\ssh",   #ie: C:\Users\Username\AppData\Roaming\ssh
+        openssh_dir,        #ie: C:\Windows\system32\OpenSSH
+        "~/.ssh",
+        "~/ssh",
+        ]
+
+def do_get_ssh_known_hosts_files():
+    #reverse the order (avoid dotfiles on win32):
+    return ("~/ssh/known_hosts", "~/.ssh/known_hosts")
+
 
 def do_get_default_conf_dirs():
     #ie: C:\Program Files\Xpra\
@@ -63,11 +101,26 @@ def do_get_default_conf_dirs():
     return [os.path.join(get_app_dir(), "etc", "xpra")]
 
 def do_get_user_conf_dirs(_uid):
-    #ie: "C:\Documents and Settings\<user name>\Application Data\Xpra" with XP
-    #or: "C:\Users\<user name>\AppData\Roaming" with Visa onwards
+    #ie: "C:\Users\<user name>\AppData\Roaming"
     return [_get_data_dir()]
 
 
+def do_get_desktop_background_paths():
+    try:
+        try:
+            import _winreg as winreg
+        except ImportError:
+            import winreg   #@UnresolvedImport @Reimport
+        key_path = "Control Panel\\Desktop"
+        key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, key_path, 0, winreg.KEY_READ)    #@UndefinedVariable
+        wallpaper = winreg.QueryValueEx(key, 'WallPaper')[0]    #@UndefinedVariable
+        return [wallpaper,]
+    except Exception:
+        log = get_util_logger()
+        log("do_get_desktop_background_paths()", exc_info=True)
+    return []
+
+
 def do_get_download_dir():
     #TODO: use "FOLDERID_Downloads":
     # FOLDERID_Downloads = "{374DE290-123F-4565-9164-39C4925E467B}"
@@ -83,8 +136,8 @@ def do_get_download_dir():
         #use the internet explorer registry key:
         #HKEY_CURRENT_USER\Software\Microsoft\Internet Explorer
         key_path = 'Software\\Microsoft\\Internet Explorer'
-        key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, key_path, 0, winreg.KEY_READ)
-        DOWNLOAD_PATH = winreg.QueryValueEx(key, 'Download Directory')[0]
+        key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, key_path, 0, winreg.KEY_READ)    #@UndefinedVariable
+        DOWNLOAD_PATH = winreg.QueryValueEx(key, 'Download Directory')[0]               #@UndefinedVariable
     except:
         #fallback to what the documentation says is the default:
         DOWNLOAD_PATH = os.path.join(os.environ.get("USERPROFILE", "~"), "My Documents", "Downloads")
@@ -97,15 +150,13 @@ def do_get_script_bin_dirs():
     return []
 
 def do_get_socket_dirs():
-    #ie: C:\Documents and Settings\Username\Application Data\Xpra
-    return [_get_data_dir()]
+    return []
 
 
 APP_DIR = None
-if getattr(sys, 'frozen', False) in (True, "windows_exe", "console_exe"):    #@UndefinedVariable
+if getattr(sys, 'frozen', False) is True:
     #cx_freeze = sys.frozen == True
-    #py2exe =  sys.frozen in ("windows_exe", "console_exe")
-    if sys.version_info >= (3,0):
+    if PYTHON3:
         APP_DIR = os.path.dirname(sys.executable)
     else:
         APP_DIR = os.path.dirname(unicode(sys.executable, sys.getfilesystemencoding()))
@@ -115,14 +166,11 @@ if getattr(sys, 'frozen', False) in (True, "windows_exe", "console_exe"):    #@U
     sys.path.insert(0, APP_DIR)
     os.chdir(APP_DIR)
     #so we can easily load DLLs with ctypes:
-    if sys.version_info >= (3,0):
+    if PYTHON3:
         os.environ['PATH'] = APP_DIR + os.pathsep + os.environ['PATH']
     else:
         os.environ['PATH'] = APP_DIR.encode('utf8') + os.pathsep + os.environ['PATH']
 
-def do_get_resources_dir():
-    from xpra.platform.paths import get_app_dir
-    return get_app_dir()
 
 def do_get_app_dir():
     global APP_DIR
@@ -134,12 +182,31 @@ def do_get_app_dir():
 def do_get_sound_command():
     from xpra.platform.paths import get_app_dir
     app_dir = get_app_dir()
-    #is there a python3 bundled sound subdirectory
-    sound_exe = os.path.join(app_dir, "Audio", "Xpra_Audio.exe")
-    #same directory, but with a nicer name:
-    if not os.path.exists(sound_exe):
-        sound_exe = os.path.join(app_dir, "Xpra_Audio.exe")
-    #fallback for older build method:
-    if not os.path.exists(sound_exe):
-        sound_exe = os.path.join(app_dir, "xpra_cmd.exe")
-    return [sound_exe]
+    for apaths in (
+        ("Audio", "Xpra_Audio.exe"),    #python3 bundled sound subdirectory
+        ("Xpra_Audio.exe",),            #same directory, but with a nicer name:
+        ("xpra_cmd.exe",),              #fallback for older build method
+        ):
+        sound_exe = os.path.join(app_dir, *apaths)
+        if os.path.exists(sound_exe):
+            return [sound_exe]
+    return do_get_xpra_command()
+
+def do_get_xpra_command():
+    from xpra.platform.paths import default_do_get_xpra_command
+    d = default_do_get_xpra_command()
+    if sys.executable.lower().endswith("xpra_cmd.exe") or sys.executable.lower().endswith("xpra.exe"):
+        return [sys.executable]
+    mingw = os.environ.get("MINGW_PREFIX")
+    if mingw:
+        xpra_script = os.path.join(os.getcwd(), "scripts", "xpra")
+        xpra_bin_script = os.path.join(mingw, "bin", "xpra")
+        py = os.path.join(mingw, "bin", "python%i.exe" % sys.version_info[0])
+        if os.path.exists(xpra_script) and os.path.exists(py):
+            return [py, xpra_script]
+        if os.path.exists(xpra_bin_script) and os.path.exists(py):
+            return [py, xpra_bin_script]
+        if len(d) == 1:
+            if not d[0].lower().endswith(".exe"):
+                return [sys.executable, d[0]]
+    return d
