@@ -1,48 +1,53 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 # Low level support for the "system tray" on MS Windows
 # Based on code from winswitch, itself based on "win32gui_taskbar demo"
 
+import os
 import ctypes
 from ctypes import POINTER, Structure, byref, WinDLL, c_void_p, sizeof, create_string_buffer
 from ctypes.wintypes import HWND, UINT, POINT, HICON, BOOL, WCHAR, DWORD
 
-from xpra.util import csv, nonl, XPRA_GUID1, XPRA_GUID2, XPRA_GUID3, XPRA_GUID4
+from xpra.util import csv, nonl, envbool, XPRA_GUID1, XPRA_GUID2, XPRA_GUID3, XPRA_GUID4
 from xpra.os_util import memoryview_to_bytes, bytestostr
 from xpra.platform.win32 import constants as win32con
-from xpra.platform.win32.common import (GUID, WNDCLASSEX, WNDPROC,
-                                        GetSystemMetrics,
-                                        GetCursorPos,
-                                        PostMessageA,
-                                        CreateWindowExA,
-                                        CreatePopupMenu, AppendMenu,
-                                        LoadIconA,
-                                        DefWindowProcA, RegisterWindowMessageA,
-                                        RegisterClassExA,
-                                        ICONINFO, BITMAPV5HEADER,
-                                        LoadImageW, CreateIconIndirect,
-                                        GetDC, ReleaseDC,
-                                        CreateBitmap, CreateDIBSection,
-                                        UpdateWindow, DestroyWindow,
-                                        PostQuitMessage,
-                                        GetModuleHandleA,
-                                        GetStockObject, DeleteObject)
+from xpra.platform.win32.common import (
+    GUID, WNDCLASSEX, WNDPROC,
+    GetSystemMetrics,
+    GetCursorPos,
+    PostMessageA,
+    CreateWindowExA, CreatePopupMenu, AppendMenu,
+    LoadIconA,
+    DefWindowProcA, RegisterWindowMessageA, RegisterClassExA,
+    ICONINFO, BITMAPV5HEADER,
+    LoadImageW, CreateIconIndirect,
+    GetDC, ReleaseDC,
+    CreateBitmap, CreateDIBSection,
+    UpdateWindow, DestroyWindow,
+    PostQuitMessage,
+    GetModuleHandleA,
+    GetStockObject, DeleteObject,
+    )
 from xpra.log import Logger
+
 log = Logger("tray", "win32")
 
 log("loading ctypes NotifyIcon functions")
 sprintf = ctypes.cdll.msvcrt.sprintf
 
 
+TRAY_ALPHA = envbool("XPRA_TRAY_ALPHA", True)
+
+
 def GetProductInfo(dwOSMajorVersion=5, dwOSMinorVersion=0, dwSpMajorVersion=0, dwSpMinorVersion=0):
     product_type = DWORD(0)
     from xpra.platform.win32.common import GetProductInfo as k32GetProductInfo
     v = k32GetProductInfo(dwOSMajorVersion, dwOSMinorVersion, dwSpMajorVersion, dwSpMinorVersion, byref(product_type))
-    log("GetProductInfo(%i, %i, %i, %i)=%i product_type=%s", dwOSMajorVersion, dwOSMinorVersion, dwSpMajorVersion, dwSpMinorVersion, v, product_type)
+    log("GetProductInfo(%i, %i, %i, %i)=%i product_type=%s", dwOSMajorVersion, dwOSMinorVersion, dwSpMajorVersion, dwSpMinorVersion, v, hex(product_type.value))
     return bool(v)
 #win7 is actually 6.1:
 try:
@@ -144,14 +149,19 @@ BUTTON_MAP = {
 def image_to_ICONINFO(img):
     w, h = img.size
     from xpra.codecs.argb.argb import rgba_to_bgra       #@UnresolvedImport
-    bgra = memoryview_to_bytes(rgba_to_bgra(img.tobytes("raw", "BGRA")))
-    return make_ICONINFO(w, h, bgra)
+    if TRAY_ALPHA and img.mode.find("A")>=0:   #ie: RGBA
+        rgb_format = "BGRA"
+    else:
+        rgb_format = "BGR"
+    rgb_data = memoryview_to_bytes(rgba_to_bgra(img.tobytes("raw", rgb_format)))
+    return make_ICONINFO(w, h, rgb_data, rgb_format=rgb_format)
 
-def make_ICONINFO(w, h, bgra):
+def make_ICONINFO(w, h, rgb_data, rgb_format="BGRA"):
     bitmap = 0
     mask = 0
     try:
-        bitmap = rgba_to_bitmap(bgra, w, h)
+        bytes_per_pixel = len(rgb_format)
+        bitmap = rgb_to_bitmap(rgb_data, bytes_per_pixel, w, h)
         mask = CreateBitmap(w, h, 1, 1, None)
 
         iconinfo = ICONINFO()
@@ -172,14 +182,14 @@ def make_ICONINFO(w, h, bgra):
         if bitmap:
             DeleteObject(bitmap)
 
-
-def rgba_to_bitmap(rgba, w, h):
+def rgb_to_bitmap(rgb_data, bytes_per_pixel, w, h):
+    assert bytes_per_pixel in (3, 4)        #only BGRA or BGR are supported
     header = BITMAPV5HEADER()
     header.bV5Size = sizeof(BITMAPV5HEADER)
     header.bV5Width = w
     header.bV5Height = -h
     header.bV5Planes = 1
-    header.bV5BitCount = 32
+    header.bV5BitCount = bytes_per_pixel*8
     header.bV5Compression = BI_RGB      #BI_BITFIELDS
     #header.bV5RedMask = 0x000000ff
     #header.bV5GreenMask = 0x0000ff00
@@ -195,8 +205,8 @@ def rgba_to_bitmap(rgba, w, h):
         ReleaseDC(None, hdc)
     assert dataptr and bitmap, "failed to create DIB section"
     log("CreateDIBSection(..) got bitmap=%#x, dataptr=%s", int(bitmap), dataptr)
-    img_data = create_string_buffer(rgba)
-    ctypes.memmove(dataptr, byref(img_data), w*4*h)
+    img_data = create_string_buffer(rgb_data)
+    ctypes.memmove(dataptr, byref(img_data), w*h*bytes_per_pixel)
     return bitmap
 
 
@@ -207,27 +217,32 @@ class win32NotifyIcon(object):
     instances = {}
 
     def __init__(self, app_id=0, title="", move_callbacks=None, click_callback=None, exit_callback=None, command_callback=None, iconPathName=None):
-        log("win32NotifyIcon: app_id=%i, title='%s'", app_id, title)
+        log("win32NotifyIcon: app_id=%i, title='%s'", app_id, nonl(title))
         self.app_id = app_id
         self.title = title
         self.current_icon = None
+        self.move_callback = move_callbacks
+        self.click_callback = click_callback
+        self.exit_callback = exit_callback
+        self.command_callback = command_callback
+        self.reset_function = None
+        self.image_cache = {}
         # Create the Window.
         if iconPathName:
             try:
                 iconPathName = iconPathName.decode()
             except:
                 pass
-            self.current_icon = self.LoadImage(iconPathName)
+            self.current_icon = self.LoadImage(iconPathName) or FALLBACK_ICON
         self.create_tray_window()
         #register callbacks:
         win32NotifyIcon.instances[self.hwnd] = self
-        self.move_callback = move_callbacks
-        self.click_callback = click_callback
-        self.exit_callback = exit_callback
-        self.command_callback = command_callback
-        self.reset_function = None
+
+    def __repr__(self):
+        return "win32NotifyIcon(%#x)" % self.app_id
 
     def create_tray_window(self):
+        log("create_tray_window()")
         self.create_window()
         self.register_tray()
 
@@ -239,7 +254,7 @@ class win32NotifyIcon(object):
             0, 0, NIwc.hInstance, None)
         if self.hwnd==0:
             raise ctypes.WinError(ctypes.get_last_error())
-        log("hwnd=%#x", self.hwnd)
+        log("create_window() hwnd=%#x", self.hwnd)
         UpdateWindow(self.hwnd)
 
     def register_tray(self):
@@ -249,6 +264,7 @@ class win32NotifyIcon(object):
             raise Exception("Shell_NotifyIconW failed to ADD")
 
     def make_nid(self, flags):
+        assert self.hwnd
         nid = NOTIFYICONDATA()
         nid.cbSize = sizeof(NOTIFYICONDATA)
         nid.hWnd = self.hwnd
@@ -280,11 +296,14 @@ class win32NotifyIcon(object):
         try:
             nid = self.make_nid(0)
             log("delete_tray_window(..) calling Shell_NotifyIconW(NIM_DELETE, %s)", nid)
-            Shell_NotifyIconW(NIM_DELETE, nid)
+            if not Shell_NotifyIconW(NIM_DELETE, nid):
+                log.warn("Warning: failed to remove system tray")
+            else:
+                self.hwnd = 0
         except Exception as e:
+            log("delete_tray_window()", exc_info=True)
             log.error("Error: failed to delete tray window")
             log.error(" %s", e)
-        self.hwnd = 0
 
 
     def set_blinking(self, on):
@@ -292,12 +311,14 @@ class win32NotifyIcon(object):
         pass
 
     def set_tooltip(self, tooltip):
+        log("set_tooltip(%s)", nonl(tooltip))
         self.title = tooltip
         Shell_NotifyIconW(NIM_MODIFY, self.make_nid(NIF_ICON | NIF_MESSAGE | NIF_TIP))
 
 
     def set_icon(self, iconPathName):
-        hicon = self.LoadImage(iconPathName)
+        log("set_icon(%s)", iconPathName)
+        hicon = self.LoadImage(iconPathName) or FALLBACK_ICON
         self.do_set_icon(hicon)
         Shell_NotifyIconW(NIM_MODIFY, self.make_nid(NIF_ICON))
         self.reset_function = (self.set_icon, iconPathName)
@@ -331,25 +352,42 @@ class win32NotifyIcon(object):
         UpdateWindow(self.hwnd)
         self.reset_function = (self.set_icon_from_data, pixels, has_alpha, w, h, rowstride)
 
-    def LoadImage(self, iconPathName, fallback=FALLBACK_ICON):
-        v = fallback
-        if iconPathName:
-            icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
-            try:
-                img_type = win32con.IMAGE_ICON
-                if iconPathName.lower().split(".")[-1] in ("png", "bmp"):
-                    img_type = win32con.IMAGE_BITMAP
-                    icon_flags |= win32con.LR_CREATEDIBSECTION | win32con.LR_LOADTRANSPARENT
-                log("LoadImage(%s) using image type=%s", iconPathName,
-                                            {
-                                                win32con.IMAGE_ICON    : "ICON",
-                                                win32con.IMAGE_BITMAP  : "BITMAP",
-                                             }.get(img_type))
-                v = LoadImageW(NIwc.hInstance, iconPathName, img_type, 0, 0, icon_flags)
-            except:
-                log.error("Failed to load icon at %s", iconPathName, exc_info=True)
-        log("LoadImage(%s)=%s", iconPathName, v)
-        return v
+    def LoadImage(self, iconPathName):
+        if not iconPathName:
+            return None
+        image = self.image_cache.get(iconPathName)
+        if not image:
+            image = self.doLoadImage(iconPathName)
+            self.image_cache[iconPathName] = image
+        return image
+
+    def doLoadImage(self, iconPathName):
+        mingw_prefix = os.environ.get("MINGW_PREFIX")
+        if mingw_prefix and iconPathName.find(mingw_prefix)>=0:
+            #python can deal with mixed win32 and unix paths,
+            #but the native win32 LoadImage function cannot,
+            #this happens when running from a mingw environment
+            iconPathName = iconPathName.replace("/", "\\")
+        icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
+        try:
+            assert os.path.exists(iconPathName)
+            img_type = win32con.IMAGE_ICON
+            if iconPathName.lower().split(".")[-1] in ("png", "bmp"):
+                img_type = win32con.IMAGE_BITMAP
+                icon_flags |= win32con.LR_CREATEDIBSECTION | win32con.LR_LOADTRANSPARENT
+            log("LoadImage(%s) using image type=%s", iconPathName,
+                                        {
+                                            win32con.IMAGE_ICON    : "ICON",
+                                            win32con.IMAGE_BITMAP  : "BITMAP",
+                                         }.get(img_type))
+            v = LoadImageW(NIwc.hInstance, iconPathName, img_type, 0, 0, icon_flags)
+            assert v is not None
+        except:
+            log.error("Error: failed to load icon '%s'", iconPathName, exc_info=True)
+            return None
+        else:
+            log("LoadImage(%s)=%#x", iconPathName, v)
+            return v
 
 
     def OnTrayRestart(self, hwnd=0, msg=0, wparam=0, lparam=0):
@@ -380,7 +418,7 @@ class win32NotifyIcon(object):
     def OnTaskbarNotify(self, hwnd, msg, wparam, lparam):
         if lparam==win32con.WM_MOUSEMOVE:
             cb = self.move_callback
-            bm = [(hwnd, msg, wparam, lparam)]
+            bm = [(hwnd, int(msg), int(wparam), int(lparam))]
         else:
             cb = self.click_callback
             bm = BUTTON_MAP.get(lparam)
@@ -408,7 +446,7 @@ class win32NotifyIcon(object):
         if hwnd:
             try:
                 del win32NotifyIcon.instances[hwnd]
-            except:
+            except KeyError:
                 pass
 
 
@@ -423,7 +461,7 @@ message_map = {
 def NotifyIconWndProc(hwnd, msg, wParam, lParam):
     instance = win32NotifyIcon.instances.get(hwnd)
     fn = message_map.get(msg)
-    log("NotifyIconWndProc%s instance=%s, message(%i)=%s", (hwnd, msg, wParam, lParam), instance, msg, fn)
+    log("NotifyIconWndProc%s instance=%s, message(%i)=%s", (int(hwnd), int(msg), int(wParam), int(lParam)), instance, msg, fn)
     #log("potential matching win32 constants for message: %s", [x for x in dir(win32con) if getattr(win32con, x)==msg])
     if instance and fn:
         return fn(instance, hwnd, msg, wParam, lParam) or 0
@@ -438,13 +476,12 @@ NIwc.hBrush = GetStockObject(win32con.WHITE_BRUSH)
 NIwc.lpszClassName = u"win32NotifyIcon"
 
 NIclassAtom = RegisterClassExA(byref(NIwc))
+log("RegisterClassExA(%s)=%i", NIwc.lpszClassName, NIclassAtom)
 if NIclassAtom==0:
     raise ctypes.WinError(ctypes.get_last_error())
-log("RegisterClassExA(%s)=%i", NIwc.lpszClassName, NIclassAtom)
 
 
 def main():
-    import os
     from xpra.platform.win32.common import user32
 
     def click_callback(button, pressed):
@@ -487,7 +524,7 @@ def main():
 
     from xpra.platform.paths import get_app_dir
     idir = os.path.abspath(get_app_dir())
-    wdir = os.path.join(idir, "win32")
+    wdir = os.path.join(idir, "icons")
     if os.path.exists(wdir):
         idir = wdir
     iconPathName = os.path.join(idir, "xpra.ico")
