@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -12,16 +12,18 @@ import os
 import platform as python_platform
 assert python_platform
 from xpra.util import csv
-from xpra.os_util import (WIN32, OSX, PYTHON2, osexpand, getuid, getgid, get_username_for_uid,
-    is_CentOS, is_RedHat, is_Fedora, is_Debian, is_Ubuntu, getUbuntuVersion)
+from xpra.os_util import (
+    WIN32, OSX, PYTHON2, osexpand, getuid, getgid, get_username_for_uid,
+    is_CentOS, is_RedHat, is_Fedora, is_Debian, is_Ubuntu, getUbuntuVersion,
+    )
 
 def warn(msg):
     sys.stderr.write(msg+"\n")
 
-def debug(*args):
+def nodebug(*_args):
     #can be overriden
     pass
-
+debug = nodebug
 
 class InitException(Exception):
     pass
@@ -33,6 +35,8 @@ class InitExit(Exception):
         Exception.__init__(self, msg)
 
 
+DEBUG_CONFIG_PROPERTIES = os.environ.get("XPRA_DEBUG_CONFIG_PROPERTIES", "").split()
+
 DEFAULT_XPRA_CONF_FILENAME = os.environ.get("XPRA_CONF_FILENAME", 'xpra.conf')
 DEFAULT_NET_WM_NAME = os.environ.get("XPRA_NET_WM_NAME", "Xpra")
 
@@ -63,6 +67,7 @@ def get_xorg_bin():
               "/usr/libexec/Xorg",              #fedora 22+
               "/usr/lib/xorg/Xorg",             #ubuntu 16.10
               "/usr/lib/xorg-server/Xorg",      #arch linux
+              "/usr/lib/Xorg",                  #arch linux (new 2019)
               "/usr/X11/bin/X",                 #OSX
               ):
         if os.path.exists(p):
@@ -157,14 +162,13 @@ def detect_xvfb_command(conf_dir="/etc/xpra/", bin_dir=None, Xdummy_ENABLED=None
 
     if Xdummy_ENABLED is False:
         return get_Xvfb_command()
-    elif Xdummy_ENABLED is True:
+    if Xdummy_ENABLED is True:
         return Xorg_suid_check()
-    else:
-        debug("Xdummy support unspecified, will try to detect")
+    debug("Xdummy support unspecified, will try to detect")
 
     if is_Ubuntu():
         rnum = getUbuntuVersion()
-        if rnum==[16, 10]:
+        if rnum==(16, 10):
             return Xorg_suid_check()
         debug("Warning: Ubuntu breaks Xorg/Xdummy usage - using Xvfb fallback")
         return get_Xvfb_command()
@@ -186,23 +190,16 @@ def OpenGL_safety_check():
         try:
             f = None
             f = open("\\\\.\\VBoxMiniRdrDN", "r")
+            return True, "VirtualBox is present (VBoxMiniRdrDN)"
         finally:
             if f:
                 f.close()
-                return True, "VirtualBox is present (VBoxMiniRdrDN)"
     except Exception as e:
         import errno
         if e.args[0]==errno.EACCES:
             return "VirtualBox is present (VBoxMiniRdrDN)"
     return None
 
-OPENGL_DEFAULT = "auto"       #will auto-detect by probing
-def get_opengl_default():
-    global OPENGL_DEFAULT
-    if OpenGL_safety_check() is not None:
-        OPENGL_DEFAULT = "no"
-    return OPENGL_DEFAULT
-
 
 def get_build_info():
     info = []
@@ -210,7 +207,7 @@ def get_build_info():
         from xpra.src_info import REVISION, LOCAL_MODIFICATIONS             #@UnresolvedImport
         try:
             mods = int(LOCAL_MODIFICATIONS)
-        except:
+        except ValueError:
             mods = 0
         if mods==0:
             info.append("revision %s" % REVISION)
@@ -219,11 +216,19 @@ def get_build_info():
     except Exception as e:
         warn("Error: could not find the source information: %s" % e)
     try:
-        from xpra.build_info import BUILT_BY, BUILT_ON, BUILD_DATE, BUILD_TIME, BUILD_BIT, CYTHON_VERSION, COMPILER_VERSION    #@UnresolvedImport
+        from xpra.build_info import (
+            BUILD_DATE, BUILD_TIME, BUILD_BIT,
+            CYTHON_VERSION, COMPILER_VERSION,
+            )
         if BUILD_BIT:
             info.insert(0, "")
             info.insert(0, BUILD_BIT)
-        info.append("built on %s by %s" % (BUILT_ON, BUILT_BY))
+        try:
+            from xpra.build_info import BUILT_BY, BUILT_ON
+            info.append("built on %s by %s" % (BUILT_ON, BUILT_BY))
+        except ImportError:
+            #reproducible builds dropped this info
+            pass
         if BUILD_DATE and BUILD_TIME:
             info.append("%s %s" % (BUILD_DATE, BUILD_TIME))
         if CYTHON_VERSION!="unknown" or COMPILER_VERSION!="unknown":
@@ -273,7 +278,7 @@ def read_config(conf_file):
         for line in f:
             sline = line.strip().rstrip('\r\n').strip()
             no += 1
-            if len(sline) == 0:
+            if not sline:
                 debug("%4s empty line", no)
                 continue
             if sline[0] in ( '!', '#' ):
@@ -292,7 +297,7 @@ def read_config(conf_file):
             l += line
             agg_lines.append(l)
             l = ""
-    if len(l)>0:
+    if l:
         #last line had a trailing backslash... meh
         agg_lines.append(l)
     debug("loaded %s aggregated lines", len(agg_lines))
@@ -307,7 +312,7 @@ def read_config(conf_file):
         value = props[1].strip()
         current_value = d.get(name)
         if current_value:
-            if type(current_value)==list:
+            if isinstance(current_value, list):
                 d[name] = current_value + [value]
             else:
                 d[name] = [current_value, value]
@@ -315,6 +320,8 @@ def read_config(conf_file):
         else:
             debug("assigned (new): %s='%s'", name, value)
             d[name] = value
+        if name in DEBUG_CONFIG_PROPERTIES:
+            print("%s=%s (was %s), from %s" % (name, d[name], current_value, conf_file))
     return  d
 
 
@@ -474,6 +481,7 @@ OPTION_TYPES = {
                     "log-file"          : str,
                     "border"            : str,
                     "window-close"      : str,
+                    "min-size"          : str,
                     "max-size"          : str,
                     "desktop-scaling"   : str,
                     "display"           : str,
@@ -488,6 +496,7 @@ OPTION_TYPES = {
                     "postscript-printer": str,
                     "debug"             : str,
                     "input-method"      : str,
+                    "video-scaling"     : str,
                     "microphone"        : str,
                     "speaker"           : str,
                     "sound-source"      : str,
@@ -527,7 +536,6 @@ OPTION_TYPES = {
                     "min-speed"         : int,
                     "compression_level" : int,
                     "dpi"               : int,
-                    "video-scaling"     : int,
                     "file-size-limit"   : int,
                     "idle-timeout"      : int,
                     "server-idle-timeout" : int,
@@ -549,7 +557,7 @@ OPTION_TYPES = {
                     "tray"              : bool,
                     "pulseaudio"        : bool,
                     "dbus-proxy"        : bool,
-                    "mmap-group"        : bool,
+                    "mmap-group"        : str,
                     "readonly"          : bool,
                     "keyboard-sync"     : bool,
                     "cursors"           : bool,
@@ -574,6 +582,9 @@ OPTION_TYPES = {
                     "desktop-fullscreen": bool,
                     "global-menus"      : bool,
                     "forward-xdg-open"  : bool,
+                    "modal-windows"     : bool,
+                    "bandwidth-detection" : bool,
+                    "ssh-upgrade"       : bool,
                     #arrays of strings:
                     "pulseaudio-configure-commands" : list,
                     "socket-dirs"       : list,
@@ -594,6 +605,8 @@ OPTION_TYPES = {
                     "start-child-after-connect" : list,
                     "start-on-connect"          : list,
                     "start-child-on-connect"    : list,
+                    "start-on-last-client-exit" : list,
+                    "start-child-on-last-client-exit"   : list,
                     "bind"              : list,
                     "bind-vsock"        : list,
                     "bind-tcp"          : list,
@@ -601,6 +614,7 @@ OPTION_TYPES = {
                     "bind-ws"           : list,
                     "bind-wss"          : list,
                     "bind-ssl"          : list,
+                    "bind-ssh"          : list,
                     "bind-rfb"          : list,
                     "auth"              : list,
                     "vsock-auth"        : list,
@@ -609,6 +623,7 @@ OPTION_TYPES = {
                     "ws-auth"           : list,
                     "wss-auth"          : list,
                     "ssl-auth"          : list,
+                    "ssh-auth"          : list,
                     "rfb-auth"          : list,
                     "password-file"     : list,
                     "start-env"         : list,
@@ -627,14 +642,18 @@ START_COMMAND_OPTIONS = [
     "start", "start-child",
     "start-after-connect", "start-child-after-connect",
     "start-on-connect", "start-child-on-connect",
+    "start-on-last-client-exit", "start-child-on-last-client-exit",
     ]
 BIND_OPTIONS = ["bind", "bind-tcp", "bind-udp", "bind-ssl", "bind-ws", "bind-wss", "bind-vsock", "bind-rfb"]
 
 #keep track of the options added since v1,
 #so we can generate command lines that work with older supported versions:
-OPTIONS_ADDED_SINCE_V1 = ["attach", "open-files", "open-url", "pixel-depth", "uid", "gid", "chdir", "min-port", "rfb-upgrade", "bandwidth-limit",
-                    "forward-xdg-open",
-                          ]
+OPTIONS_ADDED_SINCE_V1 = [
+    "attach", "open-files", "open-url", "pixel-depth",
+    "uid", "gid", "chdir", "min-port", "rfb-upgrade", "bandwidth-limit",
+    "forward-xdg-open", "modal-windows", "bandwidth-detection",
+    "bind-ssh", "ssh-auth", "ssh-upgrade",
+    ]
 OPTIONS_COMPAT_NAMES = {
     "--compression_level=" : "-z"
     }
@@ -646,8 +665,9 @@ CLIENT_OPTIONS = ["title", "username", "password", "session-name",
                   "tcp-encryption", "tcp-encryption-keyfile", "encryption",  "encryption-keyfile",
                   "systemd-run", "systemd-run-args",
                   "socket-dir", "socket-dirs",
-                  "border", "window-close", "max-size", "desktop-scaling",
-                  "file-transfer", "file-size-limit", "download-path", "open-command", "open-files", "printing", "open-url",
+                  "border", "window-close", "min-size", "max-size", "desktop-scaling",
+                  "file-transfer", "file-size-limit", "download-path",
+                  "open-command", "open-files", "printing", "open-url",
                   "challenge-handlers",
                   "dbus-proxy",
                   "remote-logging",
@@ -696,15 +716,17 @@ PROXY_START_OVERRIDABLE_OPTIONS = [
     "system-tray", "sharing", "lock", "windows", "webcam", "html",
     "terminate-children", "exit-with-children", "exit-with-client",
     "av-sync", "global-menus",
-    "forward-xdg-open",
+    "forward-xdg-open", "modal-windows", "bandwidth-detection",
+    "ssh-upgrade",
     "printing", "file-transfer", "open-command", "open-files", "open-url", "start-new-commands",
     "mmap", "mmap-group", "mdns",
-    "auth", "vsock-auth", "tcp-auth", "udp-auth", "ws-auth", "wss-auth", "ssl-auth", "rfb-auth",
-    "bind", "bind-vsock", "bind-tcp", "bind-udp", "bind-ssl", "bind-ws", "bind-wss", "bind-rfb",
+    "auth", "vsock-auth", "tcp-auth", "udp-auth", "ws-auth", "wss-auth", "ssl-auth", "ssh-auth", "rfb-auth",
+    "bind", "bind-vsock", "bind-tcp", "bind-udp", "bind-ssl", "bind-ws", "bind-wss", "bind-ssh", "bind-rfb",
     "rfb-upgrade", "bandwidth-limit",
     "start", "start-child",
     "start-after-connect", "start-child-after-connect",
     "start-on-connect", "start-child-on-connect",
+    "start-on-last-client-exit", "start-child-on-last-client-exit",
     ]
 tmp = os.environ.get("XPRA_PROXY_START_OVERRIDABLE_OPTIONS", "")
 if tmp:
@@ -759,15 +781,17 @@ def get_default_systemd_run():
 def get_default_pulseaudio_command():
     if WIN32 or OSX:
         return []
-    cmd = ["pulseaudio", "--start", "-n", "--daemonize=false", "--system=false",
-                                    "--exit-idle-time=-1", "--load=module-suspend-on-idle",
-                                    "'--load=module-null-sink sink_name=\"Xpra-Speaker\" sink_properties=device.description=\"Xpra\\ Speaker\"'",
-                                    "'--load=module-null-sink sink_name=\"Xpra-Microphone\" sink_properties=device.description=\"Xpra\\ Microphone\"'",
-                                    "'--load=module-native-protocol-unix socket=$XPRA_PULSE_SERVER'",
-                                    "--load=module-dbus-protocol",
-                                    "--load=module-x11-publish",
-                                    "--log-level=2", "--log-target=stderr"]
-    if not is_Ubuntu() or getUbuntuVersion()>[16, 4]:
+    cmd = [
+        "pulseaudio", "--start", "-n", "--daemonize=false", "--system=false",
+        "--exit-idle-time=-1", "--load=module-suspend-on-idle",
+        "'--load=module-null-sink sink_name=\"Xpra-Speaker\" sink_properties=device.description=\"Xpra\\ Speaker\"'",
+        "'--load=module-null-sink sink_name=\"Xpra-Microphone\" sink_properties=device.description=\"Xpra\\ Microphone\"'",
+        "'--load=module-native-protocol-unix socket=$XPRA_PULSE_SERVER'",
+        "--load=module-dbus-protocol",
+        "--load=module-x11-publish",
+        "--log-level=2", "--log-target=stderr",
+        ]
+    if not is_Ubuntu() or getUbuntuVersion()>(16, 4):
         cmd.append("--enable-memfd=no")
     return cmd
 
@@ -779,8 +803,10 @@ def get_defaults():
     global GLOBAL_DEFAULTS
     if GLOBAL_DEFAULTS is not None:
         return GLOBAL_DEFAULTS
-    from xpra.platform.features import DEFAULT_SSH_COMMAND, OPEN_COMMAND, DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS, \
-                                        DEFAULT_ENV, CAN_DAEMONIZE, SYSTEM_PROXY_SOCKET
+    from xpra.platform.features import (
+        OPEN_COMMAND, DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS,
+        DEFAULT_ENV, CAN_DAEMONIZE, SYSTEM_PROXY_SOCKET,
+        )
     from xpra.platform.paths import get_download_dir, get_remote_run_xpra_scripts
     try:
         from xpra.platform.info import get_username
@@ -793,7 +819,7 @@ def get_defaults():
         conf_dirs.append(os.path.join(build_root, "etc", "xpra"))
     xpra_cmd = sys.argv[0]
     bin_dir = None
-    if len(sys.argv)>0:
+    if sys.argv:
         for strip in ("/usr/bin", "/bin"):
             pos = xpra_cmd.find(strip)
             if pos>=0:
@@ -809,10 +835,6 @@ def get_defaults():
         if os.path.exists(conf_dir):
             break
     xvfb = detect_xvfb_command(conf_dir, bin_dir)
-    def addtrailingslash(v):
-        if v.endswith("/"):
-            return v
-        return v+"/"
     if WIN32:
         bind_dirs = ["Main"]
     else:
@@ -850,7 +872,7 @@ def get_defaults():
                     "encryption-keyfile": "",
                     "tcp-encryption-keyfile": "",
                     "pidfile"           : "",
-                    "ssh"               : DEFAULT_SSH_COMMAND,
+                    "ssh"               : "auto",
                     "systemd-run"       : get_default_systemd_run(),
                     "systemd-run-args"  : "",
                     "system-proxy-socket" : SYSTEM_PROXY_SOCKET,
@@ -861,6 +883,7 @@ def get_defaults():
                     "log-file"          : "$DISPLAY.log",
                     "border"            : "auto,5:off",
                     "window-close"      : "auto",
+                    "min-size"          : "",
                     "max-size"          : "",
                     "desktop-scaling"   : "auto",
                     "display"           : "",
@@ -879,7 +902,7 @@ def get_defaults():
                     "html"              : "auto",
                     "socket-permissions": "600",
                     "exec-wrapper"      : "",
-                    "dbus-launch"       : "dbus-launch --close-stderr",
+                    "dbus-launch"       : "dbus-launch --sh-syntax --close-stderr",
                     "webcam"            : ["auto", "no"][OSX],
                     "mousewheel"        : "on",
                     "input-devices"     : "auto",
@@ -909,7 +932,6 @@ def get_defaults():
                     "min-speed"         : 30,
                     "compression_level" : 1,
                     "dpi"               : 0,
-                    "video-scaling"     : 1,
                     "file-size-limit"   : 100,
                     "idle-timeout"      : 0,
                     "server-idle-timeout" : 0,
@@ -921,7 +943,7 @@ def get_defaults():
                     "rfb-upgrade"       : 5,
                     "auto-refresh-delay": 0.15,
                     "daemon"            : CAN_DAEMONIZE,
-                    "start-via-proxy"   : None,
+                    "start-via-proxy"   : False,
                     "attach"            : None,
                     "use-display"       : False,
                     "fake-xinerama"     : not OSX and not WIN32,
@@ -930,9 +952,10 @@ def get_defaults():
                     "pulseaudio"        : DEFAULT_PULSEAUDIO,
                     "dbus-proxy"        : not OSX and not WIN32,
                     "mmap"              : "yes",
-                    "mmap-group"        : False,
+                    "mmap-group"        : "auto",
                     "speaker"           : ["disabled", "on"][has_sound_support()],
                     "microphone"        : ["disabled", "off"][has_sound_support()],
+                    "video-scaling"     : "auto",
                     "readonly"          : False,
                     "keyboard-sync"     : True,
                     "displayfd"         : 0,
@@ -949,18 +972,20 @@ def get_defaults():
                     "terminate-children": False,
                     "exit-with-children": False,
                     "exit-with-client"  : False,
-                    "start-after-connect": False,
                     "start-new-commands": False,
                     "proxy-start-sessions": True,
                     "av-sync"           : True,
                     "exit-ssh"          : True,
                     "dbus-control"      : not WIN32 and not OSX,
-                    "opengl"            : get_opengl_default(),
+                    "opengl"            : "probe",
                     "mdns"              : not WIN32,
                     "swap-keys"         : OSX,  #only used on osx
                     "desktop-fullscreen": False,
                     "global-menus"      : True,
                     "forward-xdg-open"  : True,
+                    "modal-windows"     : False,
+                    "bandwidth-detection" : True,
+                    "ssh-upgrade"       : True,
                     "pulseaudio-configure-commands"  : [" ".join(x) for x in DEFAULT_PULSEAUDIO_CONFIGURE_COMMANDS],
                     "socket-dirs"       : [],
                     "remote-xpra"       : get_remote_run_xpra_scripts(),
@@ -981,6 +1006,7 @@ def get_defaults():
                     "bind-ws"           : [],
                     "bind-wss"          : [],
                     "bind-ssl"          : [],
+                    "bind-ssh"          : [],
                     "bind-rfb"          : [],
                     "auth"              : [],
                     "vsock-auth"        : [],
@@ -989,6 +1015,7 @@ def get_defaults():
                     "ws-auth"           : [],
                     "wss-auth"          : [],
                     "ssl-auth"          : [],
+                    "ssh-auth"          : [],
                     "rfb-auth"          : [],
                     "password-file"     : [],
                     "start"             : [],
@@ -997,6 +1024,8 @@ def get_defaults():
                     "start-child-after-connect" : [],
                     "start-on-connect"          : [],
                     "start-child-on-connect"    : [],
+                    "start-on-last-client-exit" : [],
+                    "start-child-on-last-client-exit"   : [],
                     "start-env"         : DEFAULT_ENV,
                     "env"               : [],
                     }
@@ -1005,58 +1034,58 @@ def get_defaults():
 CLONES = {}
 
 #these options should not be specified in config files:
-NO_FILE_OPTIONS = ["daemon"]
+NO_FILE_OPTIONS = ("daemon", )
 
 
-TRUE_OPTIONS = ["yes", "true", "1", "on", True]
-FALSE_OPTIONS = ["no", "false", "0", "off", False]
+TRUE_OPTIONS = ("yes", "true", "1", "on", True)
+FALSE_OPTIONS = ("no", "false", "0", "off", False)
+OFF_OPTIONS = ("off", )
 def parse_bool(k, v, auto=None):
-    if type(v)==str:
+    if isinstance(v, str):
         v = v.lower().strip()
     if v in TRUE_OPTIONS:
         return True
-    elif v in FALSE_OPTIONS:
+    if v in FALSE_OPTIONS:
         return False
-    elif v in ["auto", None]:
+    if v in ("auto", None):
         #keep default - which may be None!
         return auto
-    else:
-        try:
-            return bool(int(v))
-        except:
-            warn("Warning: cannot parse value '%s' for '%s' as a boolean" % (v, k))
-            return None
+    try:
+        return bool(int(v))
+    except ValueError:
+        warn("Warning: cannot parse value '%s' for '%s' as a boolean" % (v, k))
+        return None
 
 def print_bool(k, v, true_str='yes', false_str='no'):
-    if type(v)==type(None):
-        return 'auto'
-    if type(v)==bool:
+    if v is None:
+        return "auto"
+    if isinstance(v, bool):
         if v:
             return true_str
         return false_str
     warn("Warning: cannot print value '%s' for '%s' as a boolean" % (v, k))
+    return ""
 
 def parse_bool_or_int(k, v):
     return parse_bool_or_number(int, k, v)
 
 def parse_bool_or_number(numtype, k, v, auto=0):
-    if type(v)==str:
+    if isinstance(v, str):
         v = v.lower()
     if v in TRUE_OPTIONS:
         return 1
-    elif v in FALSE_OPTIONS:
+    if v in FALSE_OPTIONS:
         return 0
-    else:
-        return parse_number(numtype, k, v, auto)
+    return parse_number(numtype, k, v, auto)
 
 def parse_number(numtype, k, v, auto=0):
-    if type(v)==str:
+    if isinstance(v, str):
         v = v.lower()
     if v=="auto":
         return auto
     try:
         return numtype(v)
-    except Exception as e:
+    except ValueError as e:
         warn("Warning: cannot parse value '%s' for '%s' as a type %s: %s" % (v, k, numtype, e))
         return None
 
@@ -1072,26 +1101,25 @@ def parse_with_unit(numtype, v, subunit="bps", min_value=250000):
         import re
         if not v or v in FALSE_OPTIONS:
             return 0
-        elif v=="auto":
+        if v=="auto":
             return None
-        else:
-            r = re.match('([0-9\.]*)(.*)', v)
-            assert r
-            f = float(r.group(1))
-            unit = r.group(2).lower()
-            if unit.endswith(subunit):
-                unit = unit[:-len(subunit)]     #ie: 10mbps -> 10m
-            if unit=="b":
-                pass
-            elif unit=="k":
-                f *= 1000
-            elif unit=="m":
-                f *= 1000000
-            elif unit=="g":
-                f *= 1000000000
-            if min_value is not None:
-                assert f>=min_value, "value is too low"
-            return int(f)
+        r = re.match(r'([0-9\.]*)(.*)', v)
+        assert r
+        f = float(r.group(1))
+        unit = r.group(2).lower()
+        if unit.endswith(subunit):
+            unit = unit[:-len(subunit)]     #ie: 10mbps -> 10m
+        if unit=="b":
+            pass
+        elif unit=="k":
+            f *= 1000
+        elif unit=="m":
+            f *= 1000000
+        elif unit=="g":
+            f *= 1000000000
+        if min_value is not None:
+            assert f>=min_value, "value is too low"
+        return int(f)
     except Exception as e:
         raise InitException("invalid value for %s '%s': %s" % (numtype, v, e))
 
@@ -1117,26 +1145,26 @@ def validate_config(d={}, discard=NO_FILE_OPTIONS, extras_types={}, extras_valid
             warn("Warning: invalid option: '%s'" % k)
             continue
         if vt==str:
-            if type(v)!=str:
+            if not isinstance(v, str):
                 warn("invalid value for '%s': %s (string required)" % (k, type(v)))
                 continue
         elif vt==int:
             v = parse_bool_or_number(int, k, v)
-            if v==None:
+            if v is None:
                 continue
         elif vt==float:
             v = parse_number(float, k, v)
-            if v==None:
+            if v is None:
                 continue
         elif vt==bool:
             v = parse_bool(k, v)
             if v is None:
                 continue
         elif vt==list:
-            if type(v)==str:
+            if isinstance(v, str):
                 #could just be that we specified it only once..
                 v = [v]
-            elif type(v)==list or v==None:
+            elif isinstance(v, list) or v is None:
                 #ok so far..
                 pass
             else:
@@ -1177,7 +1205,7 @@ def dict_to_validated_config(d={}, extras_defaults={}, extras_types={}, extras_v
 
 class XpraConfig(object):
     def __repr__(self):
-        return ("XpraConfig(%s)" % self.__dict__)
+        return "XpraConfig(%s)" % self.__dict__
 
 
 def fixup_debug_option(value):
@@ -1193,9 +1221,9 @@ def fixup_debug_option(value):
     return value
 
 def _csvstr(value):
-    if type(value) in (tuple, list):
+    if isinstance(value, (tuple, list)):
         return ",".join(str(x).lower().strip() for x in value if x)
-    elif type(value)==str:
+    if isinstance(value, str):
         return value.strip().lower()
     raise Exception("don't know how to convert %s to a csv list!" % type(value))
 
@@ -1204,40 +1232,50 @@ def _nodupes(s):
     return remove_dupes(x.strip().lower() for x in s.split(","))
 
 def fixup_video_all_or_none(options):
-    from xpra.codecs.video_helper import ALL_VIDEO_ENCODER_OPTIONS as aveco
-    from xpra.codecs.video_helper import ALL_CSC_MODULE_OPTIONS as acsco
-    from xpra.codecs.video_helper import ALL_VIDEO_DECODER_OPTIONS as avedo
+    #we import below, but only if we really need to access
+    #one of the lists, because those are expensive to probe
+    #(we have to load the codec, which may load other libraries)
+    #
+    #from xpra.codecs.video_helper import (
+    # ALL_VIDEO_ENCODER_OPTIONS,
+    # ALL_CSC_MODULE_OPTIONS,
+    # ALL_VIDEO_DECODER_OPTIONS,
+    # HARDWARE_ENCODER_OPTIONS,
+    #)
+    def getlist(strarg, help_txt, all_list_name):
+        if strarg=="help":
+            from xpra.codecs import video_helper
+            raise InitInfo("the following %s may be available: %s" %
+                           (help_txt, csv(getattr(video_helper, all_list_name))))
+        elif strarg=="none":
+            return []
+        elif strarg=="all":
+            from xpra.codecs import video_helper    #@Reimport
+            return getattr(video_helper, all_list_name)
+        else:
+            return [x for x in _nodupes(strarg) if x]
     vestr   = _csvstr(options.video_encoders)
     cscstr  = _csvstr(options.csc_modules)
     vdstr   = _csvstr(options.video_decoders)
     pvestr  = _csvstr(options.proxy_video_encoders)
-    def getlist(strarg, help_txt, all_list):
-        if strarg=="help":
-            raise InitInfo("the following %s may be available: %s" % (help_txt, csv(all_list)))
-        elif strarg=="none":
-            return []
-        elif strarg=="all":
-            return all_list
-        else:
-            return [x for x in _nodupes(strarg) if x]
-    options.video_encoders  = getlist(vestr,    "video encoders",   aveco)
-    options.csc_modules     = getlist(cscstr,   "csc modules",      acsco)
-    options.video_decoders  = getlist(vdstr,    "video decoders",   avedo)
-    options.proxy_video_encoders = getlist(pvestr, "proxy video encoders", avedo)
+    options.video_encoders  = getlist(vestr,    "video encoders",   "ALL_VIDEO_ENCODER_OPTIONS")
+    options.csc_modules     = getlist(cscstr,   "csc modules",      "ALL_CSC_MODULE_OPTIONS")
+    options.video_decoders  = getlist(vdstr,    "video decoders",   "ALL_VIDEO_DECODER_OPTIONS")
+    options.proxy_video_encoders = getlist(pvestr, "proxy video encoders", "HARDWARE_ENCODER_OPTIONS")
 
 def fixup_socketdirs(options, defaults):
     if not options.socket_dirs:
         from xpra.platform.paths import get_socket_dirs
         options.socket_dirs = getattr(defaults, "socket_dirs", get_socket_dirs())
-    elif type(options.socket_dirs)==str:
+    elif isinstance(options.socket_dirs, str):
         options.socket_dirs = options.socket_dirs.split(os.path.pathsep)
     else:
-        assert type(options.socket_dirs) in (list, tuple)
+        assert isinstance(options.socket_dirs, (list, tuple))
         options.socket_dirs = [v for x in options.socket_dirs for v in x.split(os.path.pathsep)]
 
 def fixup_pings(options):
     #pings used to be a boolean, True mapped to "5"
-    if type(options.pings)==int:
+    if isinstance(options.pings, int):
         return
     try:
         pings = str(options.pings).lower()
@@ -1247,11 +1285,14 @@ def fixup_pings(options):
             options.pings = 0
         else:
             options.pings = int(options.pings)
-    except:
+    except ValueError:
         options.pings = 5
 
 def fixup_encodings(options):
-    from xpra.codecs.loader import PREFERED_ENCODING_ORDER
+    try:
+        from xpra.codecs.codec_constants import PREFERED_ENCODING_ORDER
+    except ImportError:
+        return
     RENAME = {"jpg" : "jpeg"}
     if options.encoding:
         options.encoding = RENAME.get(options.encoding, options.encoding)
@@ -1301,7 +1342,7 @@ def fixup_keyboard(options):
     def p(v):
         try:
             from xpra.util import remove_dupes
-            r = remove_dupes([x.strip() for x in v.split(",")])
+            r = remove_dupes(x.strip() for x in v.split(","))
             #remove empty string if that's the only value:
             if r and len(r)==1 and r[0]=="":
                 r = []
@@ -1320,7 +1361,7 @@ def fixup_clipboard(options):
         options.clipboard_direction = "to-client"
     elif cd=="both":
         options.clipboard_direction = "both"
-    elif cd=="disabled" or cd=="none":
+    elif cd in ("disabled", "none"):
         options.clipboard_direction = "disabled"
     else:
         warn("Warning: invalid value for clipboard-direction: '%s'" % options.clipboard_direction)
@@ -1366,7 +1407,7 @@ def main():
             v = getattr(o, name_to_field(k), "")
             if ot==bool and v is None:
                 v = "Auto"
-            if type(v)==list:
+            if isinstance(v, list):
                 v = csv(str(x) for x in v)
             print("* %-32s : %s" % (k, nonl(v)))
     from xpra.platform import program_context
@@ -1376,11 +1417,12 @@ def main():
         args = list(sys.argv[1:])
         if "-v" in args or "--verbose" in sys.argv:
             global debug
-            def debug(*args):
+            def print_debug(*args):
                 print(args[0] % args[1:])
+            debug = print_debug
             args.remove("-v")
 
-        if len(args)>0:
+        if args:
             for filename in args:
                 print("")
                 print("Configuration file '%s':" % filename)
