@@ -1,40 +1,45 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2017 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import sys
-
-from xpra.gtk_common.gobject_compat import import_glib, import_gobject
-glib = import_glib()
-glib.threads_init()
-gobject = import_gobject()
-gobject.threads_init()
 from multiprocessing import Queue as MQueue, freeze_support #@UnresolvedImport
 freeze_support()
 
-from xpra.log import Logger
-log = Logger("proxy")
-authlog = Logger("proxy", "auth")
-
-
-from xpra.util import LOGIN_TIMEOUT, AUTHENTICATION_ERROR, SESSION_NOT_FOUND, SERVER_ERROR, repr_ellipsized, print_nested_dict, csv, envfloat, envbool, typedict
-from xpra.os_util import get_username_for_uid, get_groups, get_home_for_uid, bytestostr, getuid, getgid, WIN32, POSIX
+from xpra.gtk_common.gobject_compat import import_glib, import_gobject
+from xpra.util import (
+    LOGIN_TIMEOUT, AUTHENTICATION_ERROR, SESSION_NOT_FOUND, SERVER_ERROR,
+    repr_ellipsized, print_nested_dict, csv, envfloat, envbool, typedict,
+    )
+from xpra.os_util import (
+    get_username_for_uid, get_groups, get_home_for_uid, bytestostr,
+    getuid, getgid, WIN32, POSIX,
+    )
 from xpra.server.proxy.proxy_instance_process import ProxyInstanceProcess
 from xpra.server.server_core import ServerCore
 from xpra.server.control_command import ArgsControlCommand, ControlError
 from xpra.child_reaper import getChildReaper
-from xpra.scripts.config import make_defaults_struct, PROXY_START_OVERRIDABLE_OPTIONS
+from xpra.scripts.parsing import parse_bool
+from xpra.scripts.config import make_defaults_struct, PROXY_START_OVERRIDABLE_OPTIONS, OPTION_TYPES
 from xpra.scripts.main import parse_display_name, connect_to, start_server_subprocess
 from xpra.make_thread import start_thread
+from xpra.log import Logger
+
+log = Logger("proxy")
+authlog = Logger("proxy", "auth")
+
+glib = import_glib()
+gobject = import_gobject()
 
 
 PROXY_SOCKET_TIMEOUT = envfloat("XPRA_PROXY_SOCKET_TIMEOUT", "0.1")
 PROXY_WS_TIMEOUT = envfloat("XPRA_PROXY_WS_TIMEOUT", "1.0")
 assert PROXY_SOCKET_TIMEOUT>0, "invalid proxy socket timeout"
-CAN_STOP_PROXY = envbool("XPRA_CAN_STOP_PROXY", False)
+CAN_STOP_PROXY = envbool("XPRA_CAN_STOP_PROXY", getuid()!=0)
+STOP_PROXY_SOCKET_TYPES = os.environ.get("XPRA_STOP_PROXY_SOCKET_TYPES", "unix-domain,named-pipe").split(",")
 
 
 MAX_CONCURRENT_CONNECTIONS = 200
@@ -84,7 +89,8 @@ class ProxyServer(ServerCore):
 
     def init_control_commands(self):
         ServerCore.init_control_commands(self)
-        self.control_commands["stop"] = ArgsControlCommand("stop", "stops the proxy instance on the given display", self.handle_stop_command, min_args=1, max_args=1)
+        self.control_commands["stop"] = ArgsControlCommand("stop", "stops the proxy instance on the given display",
+                                                           self.handle_stop_command, min_args=1, max_args=1)
 
 
     def make_dbus_server(self):
@@ -169,8 +175,28 @@ class ProxyServer(ServerCore):
             self.send_disconnect(proto, "invalid request")
             return
         if is_req("stop"):
+            #global kill switch:
             if not CAN_STOP_PROXY:
-                self.send_disconnect(proto, "cannot stop proxy server")
+                msg = "cannot stop proxy server"
+                log.warn("Warning: %s", msg)
+                self.send_disconnect(proto, msg)
+                return
+            #verify socket type (only local connections by default):
+            try:
+                socktype = proto._conn.get_info().get("type", "unknown")
+            except:
+                socktype = "unknown"
+            if socktype not in STOP_PROXY_SOCKET_TYPES:
+                msg = "cannot stop proxy server from a '%s' connection" % socktype
+                log.warn("Warning: %s", msg)
+                log.warn(" only from: %s", csv(STOP_PROXY_SOCKET_TYPES))
+                self.send_disconnect(proto, msg)
+                return
+            #connection must be authenticated:
+            if not proto.authenticators:
+                msg = "cannot stop proxy server from unauthenticated connections"
+                log.warn("Warning: %s", msg)
+                self.send_disconnect(proto, msg)
                 return
             self._requests.add(proto)
             #send a hello back and the client should then send its "shutdown-server" packet
@@ -179,7 +205,7 @@ class ProxyServer(ServerCore):
             def force_exit_request_client():
                 try:
                     self._requests.remove(proto)
-                except:
+                except KeyError:
                     pass
                 if not proto._closed:
                     self.send_disconnect(proto, "timeout")
@@ -258,8 +284,9 @@ class ProxyServer(ServerCore):
         socket_path = None
         display = None
         sns = c.dictget("start-new-session")
-        authlog("proxy_session: displays=%s, start_sessions=%s, start-new-session=%s", displays, self._start_sessions, sns)
-        if len(displays)==0 or sns:
+        authlog("proxy_session: displays=%s, start_sessions=%s, start-new-session=%s",
+                displays, self._start_sessions, sns)
+        if not displays or sns:
             if not self._start_sessions:
                 disconnect(SESSION_NOT_FOUND, "no displays found")
                 return
@@ -274,7 +301,8 @@ class ProxyServer(ServerCore):
                 return
         if display is None:
             display = c.strget("display")
-            authlog("proxy_session: proxy-virtual-display=%s (ignored), user specified display=%s, found displays=%s", proxy_virtual_display, display, displays)
+            authlog("proxy_session: proxy-virtual-display=%s (ignored), user specified display=%s, found displays=%s",
+                    proxy_virtual_display, display, displays)
             if display==proxy_virtual_display:
                 disconnect(SESSION_NOT_FOUND, "invalid display")
                 return
@@ -366,7 +394,8 @@ class ProxyServer(ServerCore):
                 client_conn.set_active(True)
                 process = ProxyInstanceProcess(uid, gid, env_options, session_options, self._socket_dir,
                                                self.video_encoders, self.csc_modules,
-                                               client_conn, client_state, cipher, encryption_key, server_conn, c, message_queue)
+                                               client_conn, disp_desc, client_state,
+                                               cipher, encryption_key, server_conn, c, message_queue)
                 log("starting %s from pid=%s", process, os.getpid())
                 self.processes[process] = (display, message_queue)
                 process.start()
@@ -406,6 +435,20 @@ class ProxyServer(ServerCore):
                 log.warn("Warning: ignoring invalid start override")
                 log.warn(" %s=%s", k, v)
                 continue
+            try:
+                vt = OPTION_TYPES[k]
+                if vt==str:
+                    v = bytestostr(v)
+                elif vt==bool:
+                    v = parse_bool(k, v)
+                elif vt==int:
+                    v = int(v)
+                elif vt==list:
+                    v = list(bytestostr(x) for x in v)
+            except ValueError:
+                log("start_new_session: override option %s", k, exc_info=True)
+                log.warn("Warning: ignoring invalid value %s for %s (%s)", v, k, vt)
+                continue
             log("start override: %s=%s", k, v)
             if v is not None:
                 fn = k.replace("-", "_")
@@ -454,7 +497,11 @@ class ProxyServer(ServerCore):
         #hwinstaold = set_window_station("winsta0")
         def exec_command(command):
             log("exec_command(%s)", command)
-            from xpra.platform.win32.create_process_lib import Popen, CREATIONINFO, CREATION_TYPE_TOKEN, LOGON_WITH_PROFILE, CREATE_NEW_PROCESS_GROUP, STARTUPINFO
+            from xpra.platform.win32.create_process_lib import (
+                Popen,
+                CREATIONINFO, CREATION_TYPE_TOKEN,
+                LOGON_WITH_PROFILE, CREATE_NEW_PROCESS_GROUP, STARTUPINFO,
+                )
             creation_info = CREATIONINFO()
             creation_info.dwCreationType = CREATION_TYPE_TOKEN
             creation_info.dwLogonFlags = LOGON_WITH_PROFILE
@@ -504,7 +551,7 @@ class ProxyServer(ServerCore):
 
 
     def get_info(self, proto, *_args):
-        info = ServerCore.get_info(self, proto)
+        info = ServerCore.get_minimal_server_info(self)
         info.setdefault("server", {})["type"] = "Python/GLib/proxy"
         #only show more info if we have authenticated
         #as the user running the proxy server process:
@@ -512,10 +559,9 @@ class ProxyServer(ServerCore):
             sessions = []
             for authenticator in proto.authenticators:
                 auth_sessions = authenticator.get_sessions()
-                #don't add duplicates:
-                for x in auth_sessions:
-                    if x not in sessions:
-                        sessions.append(x)
+                if auth_sessions:
+                    sessions = auth_sessions
+                    break
             if sessions:
                 uid, gid = sessions[:2]
                 if not POSIX or (uid==os.getuid() and gid==os.getgid()):
